# -*- coding: utf-8 -*-
'''
Interact with Consul

https://www.consul.io

'''

# Import Python Libs
from __future__ import absolute_import

# Import 3rd-party libs
# pylint: disable=import-error,no-name-in-module,redefined-builtin
from salt.ext.six.moves.urllib.parse import urljoin as _urljoin
import salt.ext.six
import salt.ext.six.moves.http_client
# pylint: enable=import-error,no-name-in-module

# Import salt libs
import salt.utils.http

import base64
import json

import logging
log = logging.getLogger(__name__)

from salt.exceptions import SaltInvocationError

# Don't shadow built-ins.
__func_alias__ = {
    'list_': 'list'
}

__virtualname__ = 'consul'


def _get_config():
    '''
    Retrieve Consul configuration
    '''
    return __salt__['config.get']('consul.url') or \
        __salt__['config.get']('consul:url')


def _query(function,
           consul_url,
           method='GET',
           api_version='v1',
           data=None,
           query_params=None):
    '''
    Consul object method function to construct and execute on the API URL.

    :param api_url:     The Consul api url.
    :param api_version  The Consul api version
    :param function:    The Consul api function to perform.
    :param method:      The HTTP method, e.g. GET or POST.
    :param data:        The data to be sent for POST method.
    :return:            The json response from the API call or False.
    '''
    headers = {}
    if not query_params:
        query_params = {}

    ret = {'data': '',
           'res': True}

    base_url = _urljoin(consul_url, '{0}/'.format(api_version))
    url = _urljoin(base_url, function, False)

    if data is None:
        data = {}
    data = json.dumps(data)

    result = salt.utils.http.query(
        url,
        method=method,
        params=query_params,
        data=data,
        decode=True,
        status=True,
        header_dict=headers,
        opts=__opts__,
    )

    if result.get('status', None) == salt.ext.six.moves.http_client.OK:
        ret['data'] = result.get('dict', result)
        ret['res'] = True
    elif result.get('status', None) == salt.ext.six.moves.http_client.NO_CONTENT:
        ret['res'] = False
    elif result.get('status', None) == salt.ext.six.moves.http_client.NOT_FOUND:
        ret['data'] = 'Key not found.'
        ret['res'] = False
    else:
        result = result.json()
        if result:
            ret['data'] = result
            ret['res'] = True
        else:
            ret['res'] = False
    return ret


def list_(consul_url=None, key=None, **kwargs):
    '''
    List keys in Consul

    :param consul_url: The Consul server URL.
    :param key: The key to use as the starting point for the list.
    :return: The list of keys.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.list

        salt '*' consul.list key='web'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    query_params = {}

    if 'recurse' in kwargs:
        query_params['recurse'] = 'True'

    # No key so recurse and show all values
    if not key:
        query_params['recurse'] = 'True'
        function = 'kv/'
    else:
        function = 'kv/{0}'.format(key)

    query_params['keys'] = 'True'

    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def get(consul_url=None, key=None, recurse=False, decode=False, raw=False):
    '''
    Get key from Consul

    :param consul_url: The Consul server URL.
    :param key: The key to use as the starting point for the list.
    :param recurse: Return values recursively beginning at the value of key.
    :param decode: By default values are stored as Base64 encoded values,
                   decode will return the whole key with the value decoded.
    :param raw: Simply return the decoded value of the key.
    :return: The keys in Consul.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.get key='web/key1'

        salt '*' consul.list key='web' recurse='True

        salt '*' consul.list key='web' recurse='True' decode='True'

    By default values stored in Consul are base64 encoded, passing the
    decode option will show them as the decoded values.

    .. code-block:: bash

        salt '*' consul.list key='web' recurse='True' decode='True' raw='True'

    By default Consult will return other information about the key, the raw
    option will return only the raw value.

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not key:
        raise SaltInvocationError('Required argument "key" is missing.')

    query_params = {}
    function = 'kv/{0}'.format(key)
    if recurse:
        query_params['recurse'] = 'True'
    if raw:
        query_params['raw'] = True
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)

    if ret['res']:
        if decode:
            for item in ret['data']:
                item['Value'] = base64.b64decode(item['Value'])
    return ret


def put(consul_url=None, key=None, value=None, **kwargs):
    '''
    Put values into Consul

    :param consul_url: The Consul server URL.
    :param key: The key to use as the starting point for the list.
    :param value: The value to set the key to.
    :param flags: This can be used to specify an unsigned value
                  between 0 and 2^64-1. Clients can choose to use
                  this however makes sense for their application.
    :param cas: This flag is used to turn the PUT into a
                Check-And-Set operation.
    :param acquire: This flag is used to turn the PUT into a
                    lock acquisition operation.
    :param release: This flag is used to turn the PUT into a
                    lock release operation.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.put key='web/key1' value="Hello there"

        salt '*' consul.put key='web/key1' value="Hello there"
                                acquire='d5d371f4-c380-5280-12fd-8810be175592'

        salt '*' consul.put key='web/key1' value="Hello there"
                                release='d5d371f4-c380-5280-12fd-8810be175592'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not key:
        raise SaltInvocationError('Required argument "key" is missing.')

    query_params = {}

    available_sessions = session_list(consul_url=consul_url, return_list=True)
    _current = get(consul_url=consul_url, key=key)

    if 'flags' in kwargs:
        if not kwargs['flags'] >= 0 and not kwargs['flags'] <= 2**64:
            query_params['flags'] = kwargs['flags']

    if 'cas' in kwargs:
        if _current['res']:
            if kwargs['cas'] == 0:
                ret['message'] = ('Key {0} exists, index '
                                  'must be non-zero.'.format(key))
                ret['res'] = False
                return ret

            if kwargs['cas'] != _current['data']['ModifyIndex']:
                ret['message'] = ('Key {0} exists, but indexes '
                                  'do not match.'.format(key))
                ret['res'] = False
                return ret
            query_params['cas'] = kwargs['cas']
        else:
            ret['message'] = ('Key {0} does not exists, '
                              'CAS argument can not be used.'.format(key))
            ret['res'] = False
            return ret
    else:
        log.error('Key {0} does not exist. Skipping release.')

    if 'acquire' in kwargs:
        if kwargs['acquire'] not in available_sessions:
            ret['message'] = '{0} is not a valid session.'.format(kwargs['acquire'])
            ret['res'] = False
            return ret

        query_params['acquire'] = kwargs['acquire']

    if 'release' in kwargs:
        if _current['res']:
            if 'Session' in _current['data']:
                if _current['data']['Session'] == kwargs['release']:
                    query_params['release'] = kwargs['release']
                else:
                    ret['message'] = '{0} locked by another session.'.format(key)
                    ret['res'] = False
                    return ret

            else:
                ret['message'] = '{0} is not a valid session.'.format(kwargs['acquire'])
                ret['res'] = False
        else:
            log.error('Key {0} does not exist. Skipping release.')

    data = value
    function = 'kv/{0}'.format(key)
    method = 'PUT'
    ret = _query(consul_url=consul_url,
                 function=function,
                 method=method,
                 data=data,
                 query_params=query_params)

    if ret['res']:
        ret['res'] = True
        ret['data'] = 'Added key {0} with value {1}.'.format(key, value)
    else:
        ret['res'] = False
        ret['data'] = 'Unable to add key {0} with value {1}.'.format(key, value)
    return ret


def delete(consul_url=None, key=None, **kwargs):
    '''
    Delete values from Consul

    :param consul_url: The Consul server URL.
    :param key: The key to use as the starting point for the list.
    :param recurse: Delete values recursively beginning at the value of key.
    :param cas: This flag is used to turn the DELETE into
                a Check-And-Set operation.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.delete key='web'

        salt '*' consul.delete key='web' recurse='True'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not key:
        raise SaltInvocationError('Required argument "key" is missing.')

    query_params = {}

    if 'recurse' in kwargs:
        query_params['recurse'] = True

    if 'cas' in kwargs:
        if kwargs['cas'] > 0:
            query_params['cas'] = kwargs['cas']
        else:
            ret['message'] = ('Check and Set Operation ',
                              'value must be greater than 0.')
            ret['res'] = False
            return ret

    function = 'kv/{0}'.format(key)
    ret = _query(consul_url=consul_url,
                 function=function,
                 method='DELETE',
                 query_params=query_params)

    if ret['res']:
        ret['res'] = True
        ret['message'] = 'Deleted key {0}.'.format(key)
    else:
        ret['res'] = False
        ret['message'] = 'Unable to delete key {0}.'.format(key)
    return ret


def agent_checks(consul_url=None):
    '''
    Returns the checks the local agent is managing

    :param consul_url: The Consul server URL.
    :return: Returns the checks the local agent is managing

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_checks

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    function = 'agent/checks'
    ret = _query(consul_url=consul_url,
                 function=function,
                 method='GET')
    return ret


def agent_services(consul_url=None):
    '''
    Returns the services the local agent is managing

    :param consul_url: The Consul server URL.
    :return: Returns the services the local agent is managing

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_services

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    function = 'agent/services'
    ret = _query(consul_url=consul_url,
                 function=function,
                 method='GET')
    return ret


def agent_members(consul_url=None, **kwargs):
    '''
    Returns the members as seen by the local serf agent

    :param consul_url: The Consul server URL.
    :return: Returns the members as seen by the local serf agent

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_members

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'wan' in kwargs:
        query_params['wan'] = kwargs['wan']

    function = 'agent/members'
    ret = _query(consul_url=consul_url,
                 function=function,
                 method='GET',
                 query_params=query_params)
    return ret


def agent_self(consul_url=None):
    '''
    Returns the local node configuration

    :param consul_url: The Consul server URL.
    :return: Returns the local node configuration

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_self

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    function = 'agent/self'
    ret = _query(consul_url=consul_url,
                 function=function,
                 method='GET',
                 query_params=query_params)
    return ret


def agent_maintenance(consul_url=None, **kwargs):
    '''
    Manages node maintenance mode

    :param consul_url: The Consul server URL.
    :param enable: The enable flag is required.
                   Acceptable values are either true
                   (to enter maintenance mode) or
                   false (to resume normal operation).
    :param reason: If provided, its value should be a
                   text string explaining the reason for
                   placing the node into maintenance mode.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_maintenance enable='False' reason='Upgrade in progress'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'enable' in kwargs:
        query_params['enable'] = kwargs['enable']
    else:
        ret['message'] = 'Required parameter "enable" is missing.'
        ret['res'] = False
        return ret

    if 'reason' in kwargs:
        query_params['reason'] = kwargs['reason']

    function = 'agent/maintenance'
    res = _query(consul_url=consul_url,
                 function=function,
                 method='GET',
                 query_params=query_params)
    if res['res']:
        ret['res'] = True
        ret['message'] = ('Agent maintenance mode '
                          '{0}ed.'.format(kwargs['enable']))
    else:
        ret['res'] = True
        ret['message'] = 'Unable to change maintenance mode for agent.'
    return ret


def agent_join(consul_url=None, address=None, **kwargs):
    '''
    Triggers the local agent to join a node

    :param consul_url: The Consul server URL.
    :param address: The address for the agent to connect to.
    :param wan: Causes the agent to attempt to join using the WAN pool.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_join address='192.168.1.1'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not address:
        raise SaltInvocationError('Required argument "address" is missing.')

    if 'wan' in kwargs:
        query_params['wan'] = kwargs['wan']

    function = 'agent/join/{0}'.format(address)
    res = _query(consul_url=consul_url,
                 function=function,
                 method='GET',
                 query_params=query_params)
    if res['res']:
        ret['res'] = True
        ret['message'] = ('Agent maintenance mode '
                          '{0}ed.'.format(kwargs['enable']))
    else:
        ret['res'] = False
        ret['message'] = 'Unable to change maintenance mode for agent.'
    return ret


def agent_leave(consul_url=None, node=None):
    '''
    Used to instruct the agent to force a node into the left state.

    :param consul_url: The Consul server URL.
    :param node: The node the agent will force into left state
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_leave node='web1.example.com'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not node:
        raise SaltInvocationError('Required argument "node" is missing.')

    function = 'agent/force-leave/{0}'.format(node)
    res = _query(consul_url=consul_url,
                 function=function,
                 method='GET',
                 query_params=query_params)
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Node {0} put in leave state.'.format(node)
    else:
        ret['res'] = False
        ret['message'] = 'Unable to change state for {0}.'.format(node)
    return ret


def agent_check_register(consul_url=None, **kwargs):
    '''
    The register endpoint is used to add a new check to the local agent.

    :param consul_url: The Consul server URL.
    :param name: The description of what the check is for.
    :param id: The unique name to use for the check, if not
               provided 'name' is used.
    :param notes: Human readable description of the check.
    :param script: If script is provided, the check type is
                   a script, and Consul will evaluate that script
                   based on the interval parameter.
    :param http: Check will perform an HTTP GET request against
                 the value of HTTP (expected to be a URL) based
                 on the interval parameter.
    :param ttl: If a TTL type is used, then the TTL update endpoint
                must be used periodically to update the state of the check.
    :param interval: Interval at which the check should run.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_check_register name='Memory Utilization'
                script='/usr/local/bin/check_mem.py' interval='15s'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'name' in kwargs:
        data['Name'] = kwargs['name']
    else:
        raise SaltInvocationError('Required argument "name" is missing.')

    if True not in [True for item in ('script', 'http') if item in kwargs]:
        ret['message'] = 'Required parameter "script" or "http" is missing.'
        ret['res'] = False
        return ret

    if 'id' in kwargs:
        data['ID'] = kwargs['id']

    if 'notes' in kwargs:
        data['Notes'] = kwargs['notes']

    if 'script' in kwargs:
        if 'interval' not in kwargs:
            ret['message'] = 'Required parameter "interval" is missing.'
            ret['res'] = False
            return ret
        data['Script'] = kwargs['script']
        data['Interval'] = kwargs['interval']

    if 'http' in kwargs:
        if 'interval' not in kwargs:
            ret['message'] = 'Required parameter "interval" is missing.'
            ret['res'] = False
            return ret
        data['HTTP'] = kwargs['http']
        data['Interval'] = kwargs['interval']

    function = 'agent/check/register'
    res = _query(consul_url=consul_url,
                 function=function,
                 method='PUT',
                 data=data)
    if res['res']:
        ret['res'] = True
        ret['message'] = ('Check {0} added to agent.'.format(kwargs['name']))
    else:
        ret['res'] = False
        ret['message'] = 'Unable to add check to agent.'
    return ret


def agent_check_deregister(consul_url=None, checkid=None):
    '''
    The agent will take care of deregistering the check from the Catalog.

    :param consul_url: The Consul server URL.
    :param checkid: The ID of the check to deregister from Consul.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_check_deregister checkid='Memory Utilization'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not checkid:
        raise SaltInvocationError('Required argument "checkid" is missing.')

    function = 'agent/check/deregister/{0}'.format(checkid)
    res = _query(consul_url=consul_url,
                 function=function,
                 method='GET')
    if res['res']:
        ret['res'] = True
        ret['message'] = ('Check {0} removed from agent.'.format(checkid))
    else:
        ret['res'] = False
        ret['message'] = 'Unable to remove check from agent.'
    return ret


def agent_check_pass(consul_url=None, checkid=None, **kwargs):
    '''
    This endpoint is used with a check that is of the TTL type. When this
    is called, the status of the check is set to passing and the TTL
    clock is reset.

    :param consul_url: The Consul server URL.
    :param checkid: The ID of the check to mark as passing.
    :param note: A human-readable message with the status of the check.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_check_pass checkid='redis_check1'
                note='Forcing check into passing state.'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not checkid:
        raise SaltInvocationError('Required argument "checkid" is missing.')

    if 'note' in kwargs:
        query_params['note'] = kwargs['note']

    function = 'agent/check/pass/{0}'.format(checkid)
    res = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params,
                 method='GET')
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Check {0} marked as passing.'.format(checkid)
    else:
        ret['res'] = False
        ret['message'] = 'Unable to update check {0}.'.format(checkid)
    return ret


def agent_check_warn(consul_url=None, checkid=None, **kwargs):
    '''
    This endpoint is used with a check that is of the TTL type. When this
    is called, the status of the check is set to warning and the TTL
    clock is reset.

    :param consul_url: The Consul server URL.
    :param checkid: The ID of the check to deregister from Consul.
    :param note: A human-readable message with the status of the check.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_check_warn checkid='redis_check1'
                note='Forcing check into warning state.'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not checkid:
        raise SaltInvocationError('Required argument "checkid" is missing.')

    if 'note' in kwargs:
        query_params['note'] = kwargs['note']

    function = 'agent/check/warn/{0}'.format(checkid)
    res = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params,
                 method='GET')
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Check {0} marked as warning.'.format(checkid)
    else:
        ret['res'] = False
        ret['message'] = 'Unable to update check {0}.'.format(checkid)
    return ret


def agent_check_fail(consul_url=None, checkid=None, **kwargs):
    '''
    This endpoint is used with a check that is of the TTL type. When this
    is called, the status of the check is set to critical and the
    TTL clock is reset.

    :param consul_url: The Consul server URL.
    :param checkid: The ID of the check to deregister from Consul.
    :param note: A human-readable message with the status of the check.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_check_fail checkid='redis_check1'
                note='Forcing check into critical state.'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not checkid:
        raise SaltInvocationError('Required argument "checkid" is missing.')

    if 'note' in kwargs:
        query_params['note'] = kwargs['note']

    function = 'agent/check/fail/{0}'.format(checkid)
    res = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params,
                 method='GET')
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Check {0} marked as critical.'.format(checkid)
    else:
        ret['res'] = False
        ret['message'] = 'Unable to update check {0}.'.format(checkid)
    return ret


def agent_service_register(consul_url=None, **kwargs):
    '''
    The used to add a new service, with an optional
    health check, to the local agent.

    :param consul_url: The Consul server URL.
    :param name: A name describing the service.
    :param address: The address used by the service, defaults
                    to the address of the agent.
    :param port: The port used by the service.
    :param id: Unique ID to identify the service, if not
               provided the value of the name parameter is used.
    :param tags: Identifying tags for service, string or list.
    :param script: If script is provided, the check type is
                   a script, and Consul will evaluate that script
                   based on the interval parameter.
    :param http: Check will perform an HTTP GET request against
                 the value of HTTP (expected to be a URL) based
                 on the interval parameter.
    :param check_ttl: If a TTL type is used, then the TTL update
                      endpoint must be used periodically to update
                      the state of the check.
    :param check_interval: Interval at which the check should run.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_service_register name='redis'
            tags='["master", "v1"]' address="127.0.0.1" port="8080"
            check_script="/usr/local/bin/check_redis.py" interval="10s"

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    lc_kwargs = dict()
    for k, v in salt.ext.six.iteritems(kwargs):
        lc_kwargs[k.lower()] = v

    if 'name' in lc_kwargs:
        data['Name'] = lc_kwargs['name']
    else:
        raise SaltInvocationError('Required argument "name" is missing.')

    if 'address' in lc_kwargs:
        data['Address'] = lc_kwargs['address']

    if 'port' in lc_kwargs:
        data['Port'] = lc_kwargs['port']

    if 'id' in lc_kwargs:
        data['ID'] = lc_kwargs['id']

    if 'tags' in lc_kwargs:
        _tags = lc_kwargs['tags']
        if not isinstance(_tags, list):
            _tags = [_tags]
        data['Tags'] = _tags

    if 'enabletagoverride' in lc_kwargs:
        data['EnableTagOverride'] = lc_kwargs['enabletagoverride']

    if 'check' in lc_kwargs:
        dd = dict()
        for k, v in salt.ext.six.iteritems(lc_kwargs['check']):
            dd[k.lower()] = v
        interval_required = False
        check_dd = dict()

        if 'script' in dd:
            interval_required = True
            check_dd['Script'] = dd['script']
        if 'http' in dd:
            interval_required = True
            check_dd['HTTP'] = dd['http']
        if 'ttl' in dd:
            check_dd['TTL'] = dd['ttl']
        if 'interval' in dd:
            check_dd['Interval'] = dd['interval']

        if interval_required:
            if 'Interval' not in check_dd:
                ret['message'] = 'Required parameter "interval" is missing.'
                ret['res'] = False
                return ret
        else:
            if 'Interval' in check_dd:
                del check_dd['Interval']  # not required, so ignore it

        if len(check_dd) > 0:
            data['Check'] = check_dd  # if empty, ignore it

    function = 'agent/service/register'
    res = _query(consul_url=consul_url,
                 function=function,
                 method='PUT',
                 data=data)
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Service {0} registered on agent.'.format(kwargs['name'])
    else:
        ret['res'] = False
        ret['message'] = 'Unable to register service {0}.'.format(kwargs['name'])
    return ret


def agent_service_deregister(consul_url=None, serviceid=None):
    '''
    Used to remove a service.

    :param consul_url: The Consul server URL.
    :param serviceid: A serviceid describing the service.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_service_deregister serviceid='redis'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not serviceid:
        raise SaltInvocationError('Required argument "serviceid" is missing.')

    function = 'agent/service/deregister/{0}'.format(serviceid)
    res = _query(consul_url=consul_url,
                 function=function,
                 method='PUT',
                 data=data)
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Service {0} removed from agent.'.format(serviceid)
    else:
        ret['res'] = False
        ret['message'] = 'Unable to remove service {0}.'.format(serviceid)
    return ret


def agent_service_maintenance(consul_url=None, serviceid=None, **kwargs):
    '''
    Used to place a service into maintenance mode.

    :param consul_url: The Consul server URL.
    :param serviceid: A name of the service.
    :param enable: Whether the service should be enabled or disabled.
    :param reason: A human readable message of why the service was
                   enabled or disabled.
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.agent_service_deregister serviceid='redis'
                enable='True' reason='Down for upgrade'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not serviceid:
        raise SaltInvocationError('Required argument "serviceid" is missing.')

    if 'enable' in kwargs:
        query_params['enable'] = kwargs['enable']
    else:
        ret['message'] = 'Required parameter "enable" is missing.'
        ret['res'] = False
        return ret

    if 'reason' in kwargs:
        query_params['reason'] = kwargs['reason']

    function = 'agent/service/maintenance/{0}'.format(serviceid)
    res = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)

    if res['res']:
        ret['res'] = True
        ret['message'] = ('Service {0} set in '
                          'maintenance mode.'.format(serviceid))
    else:
        ret['res'] = False
        ret['message'] = ('Unable to set service '
                          '{0} to maintenance mode.'.format(serviceid))
    return ret


def session_create(consul_url=None, **kwargs):
    '''
    Used to create a session.

    :param consul_url: The Consul server URL.
    :param lockdelay: Duration string using a "s" suffix for seconds.
                      The default is 15s.
    :param node: Must refer to a node that is already registered,
                 if specified. By default, the agent's own node
                 name is used.
    :param name: A human-readable name for the session
    :param checks: A list of associated health checks. It is highly
                   recommended that, if you override this list, you
                   include the default "serfHealth".
    :param behavior: Can be set to either release or delete. This controls
                     the behavior when a session is invalidated. By default,
                     this is release, causing any locks that are held to be
                     released. Changing this to delete causes any locks that
                     are held to be deleted. delete is useful for creating
                     ephemeral key/value entries.
    :param ttl: Session is invalidated if it is not renewed before
                the TTL expires
    :return: Boolean and message indicating success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.session_create node='node1' name='my-session'
                behavior='delete' ttl='3600s'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret
    data = {}

    if 'lockdelay' in kwargs:
        data['LockDelay'] = kwargs['lockdelay']

    if 'node' in kwargs:
        data['Node'] = kwargs['node']

    if 'name' in kwargs:
        data['Name'] = kwargs['name']
    else:
        raise SaltInvocationError('Required argument "name" is missing.')

    if 'checks' in kwargs:
        data['Touch'] = kwargs['touch']

    if 'behavior' in kwargs:
        if not kwargs['behavior'] in ('delete', 'release'):
            ret['message'] = ('Behavior must be ',
                              'either delete or release.')
            ret['res'] = False
            return ret
        data['Behavior'] = kwargs['behavior']

    if 'ttl' in kwargs:
        _ttl = kwargs['ttl']
        if str(_ttl).endswith('s'):
            _ttl = _ttl[:-1]

        if int(_ttl) < 0 or int(_ttl) > 3600:
            ret['message'] = ('TTL must be ',
                              'between 0 and 3600.')
            ret['res'] = False
            return ret
        data['TTL'] = '{0}s'.format(_ttl)

    function = 'session/create'
    res = _query(consul_url=consul_url,
                 function=function,
                 method='PUT',
                 data=data)

    if res['res']:
        ret['res'] = True
        ret['message'] = 'Created session {0}.'.format(kwargs['name'])
    else:
        ret['res'] = False
        ret['message'] = 'Unable to create session {0}.'.format(kwargs['name'])
    return ret


def session_list(consul_url=None, return_list=False, **kwargs):
    '''
    Used to list sessions.

    :param consul_url: The Consul server URL.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :param return_list: By default, all information about the sessions is
                        returned, using the return_list parameter will return
                        a list of session IDs.
    :return: A list of all available sessions.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.session_list

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    query_params = {}

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'session/list'
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)

    if return_list:
        _list = []
        for item in ret['data']:
            _list.append(item['ID'])
        return _list
    return ret


def session_destroy(consul_url=None, session=None, **kwargs):
    '''
    Destroy session

    :param consul_url: The Consul server URL.
    :param session: The ID of the session to destroy.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.session_destroy session='c1c4d223-91cb-3d1f-1ee8-f2af9e7b6716'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not session:
        raise SaltInvocationError('Required argument "session" is missing.')

    query_params = {}

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'session/destroy/{0}'.format(session)
    res = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Created Service {0}.'.format(kwargs['name'])
    else:
        ret['res'] = False
        ret['message'] = 'Unable to create service {0}.'.format(kwargs['name'])
    return ret


def session_info(consul_url=None, session=None, **kwargs):
    '''
    Information about a session

    :param consul_url: The Consul server URL.
    :param session: The ID of the session to return information about.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.session_info session='c1c4d223-91cb-3d1f-1ee8-f2af9e7b6716'

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not session:
        raise SaltInvocationError('Required argument "session" is missing.')

    query_params = {}

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'session/info/{0}'.format(session)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def catalog_register(consul_url=None, **kwargs):
    '''
    Registers a new node, service, or check

    :param consul_url: The Consul server URL.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :param node: The node to register.
    :param address: The address of the node.
    :param service: The service that will be registered.
    :param service_address: The address that the service listens on.
    :param service_port: The port for the service.
    :param service_id: A unique identifier for the service, if this is not
                       provided "name" will be used.
    :param service_tags: Any tags associated with the service.
    :param check: The name of the health check to register
    :param check_status: The initial status of the check,
                         must be one of unknown, passing, warning, or critical.
    :param check_service: The service that the check is performed against.
    :param check_id: Unique identifier for the service.
    :param check_notes: An opaque field that is meant to hold human-readable text.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_register node='node1' address='192.168.1.1'
            service='redis' service_address='127.0.0.1' service_port='8080'
            service_id='redis_server1'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'datacenter' in kwargs:
        data['Datacenter'] = kwargs['datacenter']

    if 'node' in kwargs:
        data['Node'] = kwargs['node']
    else:
        ret['message'] = 'Required argument node argument is missing.'
        ret['res'] = False
        return ret

    if 'address' in kwargs:
        data['Address'] = kwargs['address']
    else:
        ret['message'] = 'Required argument address argument is missing.'
        ret['res'] = False
        return ret

    if 'service' in kwargs:
        data['Service'] = {}
        data['Service']['Service'] = kwargs['service']

        if 'service_address' in kwargs:
            data['Service']['Address'] = kwargs['service_address']

        if 'service_port' in kwargs:
            data['Service']['Port'] = kwargs['service_port']

        if 'service_id' in kwargs:
            data['Service']['ID'] = kwargs['service_id']

        if 'service_tags' in kwargs:
            _tags = kwargs['service_tags']
            if not isinstance(_tags, list):
                _tags = [_tags]
            data['Service']['Tags'] = _tags

    if 'check' in kwargs:
        data['Check'] = {}
        data['Check']['Name'] = kwargs['check']

        if 'check_status' in kwargs:
            if kwargs['check_status'] not in ('unknown', 'passing', 'warning', 'critical'):
                ret['message'] = 'Check status must be unknown, passing, warning, or critical.'
                ret['res'] = False
                return ret
            data['Check']['Status'] = kwargs['check_status']

        if 'check_service' in kwargs:
            data['Check']['ServiceID'] = kwargs['check_service']

        if 'check_id' in kwargs:
            data['Check']['CheckID'] = kwargs['check_id']

        if 'check_notes' in kwargs:
            data['Check']['Notes'] = kwargs['check_notes']

    function = 'catalog/register'
    res = _query(consul_url=consul_url,
                 function=function,
                 method='PUT',
                 data=data)
    if res['res']:
        ret['res'] = True
        ret['message'] = ('Catalog registration '
                          'for {0} successful.'.format(kwargs['node']))
    else:
        ret['res'] = False
        ret['message'] = ('Catalog registration '
                          'for {0} failed.'.format(kwargs['node']))
    return ret


def catalog_deregister(consul_url=None, **kwargs):
    '''
    Deregisters a node, service, or check

    :param consul_url: The Consul server URL.
    :param node: The node to deregister.
    :param datacenter: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :param checkid: The ID of the health check to deregister.
    :param serviceid: The ID of the service to deregister.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_register node='node1'
            serviceid='redis_server1' checkid='redis_check1'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'datacenter' in kwargs:
        data['Datacenter'] = kwargs['datacenter']

    if 'node' in kwargs:
        data['Node'] = kwargs['node']
    else:
        ret['message'] = 'Node argument required.'
        ret['res'] = False
        return ret

    if 'checkid' in kwargs:
        data['CheckID'] = kwargs['checkid']

    if 'serviceid' in kwargs:
        data['ServiceID'] = kwargs['serviceid']

    function = 'catalog/deregister'
    res = _query(consul_url=consul_url,
                 function=function,
                 method='PUT',
                 data=data)
    if res['res']:
        ret['res'] = True
        ret['message'] = 'Catalog item {0} removed.'.format(kwargs['node'])
    else:
        ret['res'] = False
        ret['message'] = ('Removing Catalog '
                          'item {0} failed.'.format(kwargs['node']))
    return ret


def catalog_datacenters(consul_url=None):
    '''
    Return list of available datacenters from catalog.

    :param consul_url: The Consul server URL.
    :return: The list of available datacenters.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_datacenters

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    function = 'catalog/datacenters'
    ret = _query(consul_url=consul_url,
                 function=function)
    return ret


def catalog_nodes(consul_url=None, **kwargs):
    '''
    Return list of available nodes from catalog.

    :param consul_url: The Consul server URL.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: The list of available nodes.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_nodes

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'catalog/nodes'
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def catalog_services(consul_url=None, **kwargs):
    '''
    Return list of available services rom catalog.

    :param consul_url: The Consul server URL.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: The list of available services.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_services

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'catalog/services'
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def catalog_service(consul_url=None, service=None, **kwargs):
    '''
    Information about the registered service.

    :param consul_url: The Consul server URL.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :param tag: Filter returned services with tag parameter.
    :return: Information about the requested service.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_service service='redis'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not service:
        raise SaltInvocationError('Required argument "service" is missing.')

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    if 'tag' in kwargs:
        query_params['tag'] = kwargs['tag']

    function = 'catalog/service/{0}'.format(service)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def catalog_node(consul_url=None, node=None, **kwargs):
    '''
    Information about the registered node.

    :param consul_url: The Consul server URL.
    :param node: The node to request information about.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: Information about the requested node.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.catalog_service service='redis'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not node:
        raise SaltInvocationError('Required argument "node" is missing.')

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'catalog/node/{0}'.format(node)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def health_node(consul_url=None, node=None, **kwargs):
    '''
    Health information about the registered node.

    :param consul_url: The Consul server URL.
    :param node: The node to request health information about.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: Health information about the requested node.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.health_node node='node1'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not node:
        raise SaltInvocationError('Required argument "node" is missing.')

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'health/node/{0}'.format(node)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def health_checks(consul_url=None, service=None, **kwargs):
    '''
    Health information about the registered service.

    :param consul_url: The Consul server URL.
    :param service: The service to request health information about.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: Health information about the requested node.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.health_checks service='redis1'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not service:
        raise SaltInvocationError('Required argument "service" is missing.')

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    function = 'health/checks/{0}'.format(service)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def health_service(consul_url=None, service=None, **kwargs):
    '''
    Health information about the registered service.

    :param consul_url: The Consul server URL.
    :param service: The service to request health information about.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :param tag: Filter returned services with tag parameter.
    :param passing: Filter results to only nodes with all
                    checks in the passing state.
    :return: Health information about the requested node.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.health_service service='redis1'

        salt '*' consul.health_service service='redis1' passing='True'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not service:
        raise SaltInvocationError('Required argument "service" is missing.')

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    if 'tag' in kwargs:
        query_params['tag'] = kwargs['tag']

    if 'passing' in kwargs:
        query_params['passing'] = kwargs['passing']

    function = 'health/service/{0}'.format(service)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def health_state(consul_url=None, state=None, **kwargs):
    '''
    Returns the checks in the state provided on the path.

    :param consul_url: The Consul server URL.
    :param state: The state to show checks for. The supported states
                  are any, unknown, passing, warning, or critical.
                  The any state is a wildcard that can be used to
                  return all checks.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :return: The checks in the provided state.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.health_state state='redis1'

        salt '*' consul.health_state service='redis1' passing='True'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not state:
        raise SaltInvocationError('Required argument "state" is missing.')

    if 'dc' in kwargs:
        query_params['dc'] = kwargs['dc']

    if state not in ('any', 'unknown', 'passing', 'warning', 'critical'):
        ret['message'] = 'State must be any, unknown, passing, warning, or critical.'
        ret['res'] = False
        return ret

    function = 'health/state/{0}'.format(state)
    ret = _query(consul_url=consul_url,
                 function=function,
                 query_params=query_params)
    return ret


def status_leader(consul_url=None):
    '''
    Returns the current Raft leader

    :param consul_url: The Consul server URL.
    :return: The address of the Raft leader.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.status_leader

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    function = 'status/leader'
    ret = _query(consul_url=consul_url,
                 function=function)
    return ret


def status_peers(consul_url):
    '''
    Returns the current Raft peer set

    :param consul_url: The Consul server URL.
    :return: Retrieves the Raft peers for the
             datacenter in which the the agent is running.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.status_peers

    '''
    ret = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    function = 'status/peers'
    ret = _query(consul_url=consul_url,
                 function=function)
    return ret


def acl_create(consul_url=None, **kwargs):
    '''
    Create a new ACL token.

    :param consul_url: The Consul server URL.
    :param name: Meaningful indicator of the ACL's purpose.
    :param type: Type is either client or management. A management
                 token is comparable to a root user and has the
                 ability to perform any action including creating,
                 modifying, and deleting ACLs.
    :param rules: The Consul server URL.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.acl_create

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'name' in kwargs:
        data['Name'] = kwargs['name']
    else:
        raise SaltInvocationError('Required argument "name" is missing.')

    if 'type' in kwargs:
        data['Type'] = kwargs['type']

    if 'rules' in kwargs:
        data['Rules'] = kwargs['rules']

    function = 'acl/create'
    res = _query(consul_url=consul_url,
                 data=data,
                 method='PUT',
                 function=function)

    if res['res']:
        ret['res'] = True
        ret['message'] = 'ACL {0} created.'.format(kwargs['name'])
    else:
        ret['res'] = False
        ret['message'] = ('Removing Catalog '
                          'item {0} failed.'.format(kwargs['name']))
    return ret


def acl_update(consul_url=None, **kwargs):
    '''
    Update an ACL token.

    :param consul_url: The Consul server URL.
    :param name: Meaningful indicator of the ACL's purpose.
    :param id: Unique identifier for the ACL to update.
    :param type: Type is either client or management. A management
                 token is comparable to a root user and has the
                 ability to perform any action including creating,
                 modifying, and deleting ACLs.
    :param rules: The Consul server URL.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.acl_update

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'id' in kwargs:
        data['ID'] = kwargs['id']
    else:
        ret['message'] = 'Required parameter "id" is missing.'
        ret['res'] = False
        return ret

    if 'name' in kwargs:
        data['Name'] = kwargs['name']
    else:
        raise SaltInvocationError('Required argument "name" is missing.')

    if 'type' in kwargs:
        data['Type'] = kwargs['type']

    if 'rules' in kwargs:
        data['Rules'] = kwargs['rules']

    function = 'acl/update'
    res = _query(consul_url=consul_url,
                 data=data,
                 method='PUT',
                 function=function)

    if res['res']:
        ret['res'] = True
        ret['message'] = 'ACL {0} created.'.format(kwargs['name'])
    else:
        ret['res'] = False
        ret['message'] = ('Adding ACL '
                          '{0} failed.'.format(kwargs['name']))

    return ret


def acl_delete(consul_url=None, **kwargs):
    '''
    Delete an ACL token.

    :param consul_url: The Consul server URL.
    :param id: Unique identifier for the ACL to update.
    :return: Boolean & message of success or failure.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.acl_delete id='c1c4d223-91cb-3d1f-1ee8-f2af9e7b6716'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'id' not in kwargs:
        ret['message'] = 'Required parameter "id" is missing.'
        ret['res'] = False
        return ret

    function = 'acl/delete/{0}'.format(kwargs['id'])
    res = _query(consul_url=consul_url,
                 data=data,
                 method='PUT',
                 function=function)

    if res['res']:
        ret['res'] = True
        ret['message'] = 'ACL {0} deleted.'.format(kwargs['id'])
    else:
        ret['res'] = False
        ret['message'] = ('Removing ACL '
                          '{0} failed.'.format(kwargs['id']))

    return ret


def acl_info(consul_url=None, **kwargs):
    '''
    Information about an ACL token.

    :param consul_url: The Consul server URL.
    :param id: Unique identifier for the ACL to update.
    :return: Information about the ACL requested.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.acl_info id='c1c4d223-91cb-3d1f-1ee8-f2af9e7b6716'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'id' not in kwargs:
        ret['message'] = 'Required parameter "id" is missing.'
        ret['res'] = False
        return ret

    function = 'acl/info/{0}'.format(kwargs['id'])
    ret = _query(consul_url=consul_url,
                 data=data,
                 method='PUT',
                 function=function)
    return ret


def acl_clone(consul_url=None, **kwargs):
    '''
    Information about an ACL token.

    :param consul_url: The Consul server URL.
    :param id: Unique identifier for the ACL to update.
    :return: Boolean, message of success or
             failure, and new ID of cloned ACL.

    CLI Example:

    .. code-block:: bash

        salt '*' consul.acl_info id='c1c4d223-91cb-3d1f-1ee8-f2af9e7b6716'

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'id' not in kwargs:
        ret['message'] = 'Required parameter "id" is missing.'
        ret['res'] = False
        return ret

    function = 'acl/clone/{0}'.format(kwargs['id'])
    res = _query(consul_url=consul_url,
                 data=data,
                 method='PUT',
                 function=function)
    if res['res']:
        ret['res'] = True
        ret['message'] = 'ACL {0} cloned.'.format(kwargs['name'])
        ret['ID'] = ret['data']
    else:
        ret['res'] = False
        ret['message'] = ('Cloning ACL'
                          'item {0} failed.'.format(kwargs['name']))
    return ret


def acl_list(consul_url=None, **kwargs):
    '''
    List the ACL tokens.

    :param consul_url: The Consul server URL.
    :return: List of ACLs

    CLI Example:

    .. code-block:: bash

        salt '*' consul.acl_list

    '''
    ret = {}
    data = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'id' not in kwargs:
        ret['message'] = 'Required parameter "id" is missing.'
        ret['res'] = False
        return ret

    function = 'acl/list'
    ret = _query(consul_url=consul_url,
                 data=data,
                 method='PUT',
                 function=function)
    return ret


def event_fire(consul_url=None, name=None, **kwargs):
    '''
    List the ACL tokens.

    :param consul_url: The Consul server URL.
    :param name: The name of the event to fire.
    :param dc: By default, the datacenter of the agent is queried;
               however, the dc can be provided using the "dc" parameter.
    :param node: Filter by node name.
    :param service: Filter by service name.
    :param tag: Filter by tag name.
    :return: List of ACLs

    CLI Example:

    .. code-block:: bash

        salt '*' consul.event_fire name='deploy'

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if not name:
        raise SaltInvocationError('Required argument "name" is missing.')

    if 'dc' in kwargs:
        query_params = kwargs['dc']

    if 'node' in kwargs:
        query_params = kwargs['node']

    if 'service' in kwargs:
        query_params = kwargs['service']

    if 'tag' in kwargs:
        query_params = kwargs['tag']

    function = 'event/fire/{0}'.format(name)
    res = _query(consul_url=consul_url,
                 query_params=query_params,
                 method='PUT',
                 function=function)

    if res['res']:
        ret['res'] = True
        ret['message'] = 'Event {0} fired.'.format(name)
        ret['data'] = ret['data']
    else:
        ret['res'] = False
        ret['message'] = ('Cloning ACL'
                          'item {0} failed.'.format(kwargs['name']))
    return ret


def event_list(consul_url=None, **kwargs):
    '''
    List the recent events.

    :param consul_url: The Consul server URL.
    :param name: The name of the event to fire.
    :return: List of ACLs

    CLI Example:

    .. code-block:: bash

        salt '*' consul.event_list

    '''
    ret = {}
    query_params = {}
    if not consul_url:
        consul_url = _get_config()
        if not consul_url:
            log.error('No Consul URL found.')
            ret['message'] = 'No Consul URL found.'
            ret['res'] = False
            return ret

    if 'name' in kwargs:
        query_params = kwargs['name']
    else:
        raise SaltInvocationError('Required argument "name" is missing.')

    function = 'event/list/'
    ret = _query(consul_url=consul_url,
                 query_params=query_params,
                 function=function)
    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import debian_ip

# Import third party libs
import jinja2.exceptions

# Globals
debian_ip.__grains__ = {}
debian_ip.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DebianIpTestCase(TestCase):
    '''
    Test cases for salt.modules.debian_ip
    '''
    # 'build_bond' function tests: 3

    @patch('salt.modules.debian_ip._parse_settings_bond',
           MagicMock(return_value={}))
    @patch('salt.modules.debian_ip._write_file', MagicMock(return_value=True))
    def test_build_bond(self):
        '''
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        '''
        mock = MagicMock(return_value=1)
        with patch.dict(debian_ip.__grains__, {'osrelease': mock}):
            mock = MagicMock(return_value=True)
            with patch.dict(debian_ip.__salt__, {'kmod.load': mock,
                                                 'pkg.install': mock}):
                self.assertEqual(debian_ip.build_bond('bond0'), '')

    @patch('salt.modules.debian_ip._parse_settings_bond',
           MagicMock(return_value={}))
    def test_build_bond_exception(self):
        '''
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        '''
        mock = MagicMock(return_value=1)
        with patch.dict(debian_ip.__grains__, {'osrelease': mock}):
            mock = MagicMock(side_effect=
                             jinja2.exceptions.TemplateNotFound('error'))
            with patch.object(jinja2.Environment, 'get_template', mock):
                self.assertEqual(debian_ip.build_bond('bond0'), '')

    @patch('salt.modules.debian_ip._parse_settings_bond',
           MagicMock(return_value={}))
    @patch('salt.modules.debian_ip._read_temp', MagicMock(return_value=True))
    def test_build_bond_data(self):
        '''
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        '''
        mock = MagicMock(return_value=1)
        with patch.dict(debian_ip.__grains__, {'osrelease': mock}):
            self.assertTrue(debian_ip.build_bond('bond0', test='True'))

    # 'build_interface' function tests: 1

    @patch('salt.modules.debian_ip._write_file_ifaces',
           MagicMock(return_value='salt'))
    def test_build_interface(self):
        '''
        Test if it builds an interface script for a network interface.
        '''
        self.assertEqual(debian_ip.build_interface('eth0', 'eth', 'enabled'),
                         ['s\n', 'a\n', 'l\n', 't\n'])

        self.assertTrue(debian_ip.build_interface('eth0', 'eth', 'enabled',
                                                  test='True'))

        with patch.object(debian_ip, '_parse_settings_eth',
                          MagicMock(return_value={'routes': []})):
            self.assertRaises(AttributeError, debian_ip.build_interface,
                              'eth0', 'bridge', 'enabled')

            self.assertRaises(AttributeError, debian_ip.build_interface,
                              'eth0', 'slave', 'enabled')

            self.assertRaises(AttributeError, debian_ip.build_interface,
                              'eth0', 'bond', 'enabled')

        self.assertTrue(debian_ip.build_interface('eth0', 'eth', 'enabled',
                                                  test='True'))

    # 'build_routes' function tests: 2

    @patch('salt.modules.debian_ip._parse_routes',
           MagicMock(return_value={'routes': []}))
    @patch('salt.modules.debian_ip._write_file_routes',
           MagicMock(return_value=True))
    @patch('salt.modules.debian_ip._read_file', MagicMock(return_value='salt'))
    def test_build_routes(self):
        '''
        Test if it add route scripts for a network interface using up commands.
        '''
        self.assertEqual(debian_ip.build_routes('eth0'), 'saltsalt')

    @patch('salt.modules.debian_ip._parse_routes',
           MagicMock(return_value={'routes': []}))
    def test_build_routes_exception(self):
        '''
        Test if it add route scripts for a network interface using up commands.
        '''
        self.assertTrue(debian_ip.build_routes('eth0', test='True'))

        mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound('err'))
        with patch.object(jinja2.Environment, 'get_template', mock):
            self.assertEqual(debian_ip.build_routes('eth0'), '')

    # 'down' function tests: 1

    def test_down(self):
        '''
        Test if it shutdown a network interface
        '''
        self.assertEqual(debian_ip.down('eth0', 'slave'), None)

        mock = MagicMock(return_value='Salt')
        with patch.dict(debian_ip.__salt__, {'cmd.run': mock}):
            self.assertEqual(debian_ip.down('eth0', 'eth'), 'Salt')

    # 'get_bond' function tests: 1

    def test_get_bond(self):
        '''
        Test if it return the content of a bond script
        '''
        self.assertEqual(debian_ip.get_bond('bond0'), '')

    # 'get_interface' function tests: 1

    def test_get_interface(self):
        '''
        Test if it return the contents of an interface script
        '''
        with patch.object(debian_ip, '_parse_interfaces',
                          MagicMock(return_value={})):
            self.assertListEqual(debian_ip.get_interface('eth0'), [])

        mock_ret = {'lo': {'enabled': True, 'data':
                           {'inet': {'addrfam': 'inet', 'proto': 'loopback'}}}}
        with patch.object(debian_ip, '_parse_interfaces',
                          MagicMock(return_value=mock_ret)):
            self.assertListEqual(debian_ip.get_interface('lo'),
                                 [u'auto lo\n', u'iface lo inet loopback\n',
                                  u'\n'])

            mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                             ('error'))
            with patch.object(jinja2.Environment, 'get_template', mock):
                self.assertEqual(debian_ip.get_interface('lo'), '')

    # 'up' function tests: 1

    def test_up(self):
        '''
        Test if it start up a network interface
        '''
        self.assertEqual(debian_ip.down('eth0', 'slave'), None)

        mock = MagicMock(return_value='Salt')
        with patch.dict(debian_ip.__salt__, {'cmd.run': mock}):
            self.assertEqual(debian_ip.up('eth0', 'eth'), 'Salt')

    # 'get_network_settings' function tests: 1

    @patch('salt.modules.debian_ip._parse_hostname',
           MagicMock(return_value='SaltStack'))
    @patch('salt.modules.debian_ip._parse_domainname',
           MagicMock(return_value='saltstack.com'))
    def test_get_network_settings(self):
        '''
        Test if it return the contents of the global network script.
        '''
        with patch.dict(debian_ip.__grains__, {'osfullname': 'Ubuntu',
                                               'osrelease': '14'}):
            mock_avai = MagicMock(return_value=True)
            with patch.dict(debian_ip.__salt__, {'service.available': mock_avai,
                                                 'service.status': mock_avai}):
                self.assertEqual(debian_ip.get_network_settings(),
                                 [u'NETWORKING=yes\n',
                                  u'HOSTNAME=SaltStack\n',
                                  u'DOMAIN=saltstack.com\n'])

                mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                                 ('error'))
                with patch.object(jinja2.Environment, 'get_template', mock):
                    self.assertEqual(debian_ip.get_network_settings(), '')

    # 'get_routes' function tests: 1

    @patch('salt.modules.debian_ip._read_file', MagicMock(return_value='salt'))
    def test_get_routes(self):
        '''
        Test if it return the routes for the interface
        '''
        self.assertEqual(debian_ip.get_routes('eth0'), 'saltsalt')

    # 'apply_network_settings' function tests: 1

    def test_apply_network_settings(self):
        '''
        Test if it apply global network configuration.
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(debian_ip.__salt__, {'network.mod_hostname': mock,
                                             'service.stop': mock,
                                             'service.start': mock}):
            self.assertEqual(debian_ip.apply_network_settings(), True)

    # 'build_network_settings' function tests: 1

    @patch('salt.modules.debian_ip._parse_network_settings',
           MagicMock(return_value={'networking': 'yes',
                                   'hostname': 'Salt.saltstack.com',
                                   'domainname': 'saltstack.com'}))
    @patch('salt.modules.debian_ip._write_file_network',
           MagicMock(return_value=True))
    def test_build_network_settings(self):
        '''
        Test if it build the global network script.
        '''
        with patch.dict(debian_ip.__grains__, {'osfullname': 'Ubuntu',
                                               'osrelease': '14'}):
            mock = MagicMock(return_value=True)
            with patch.dict(debian_ip.__salt__, {'service.available': mock,
                                                 'service.disable': mock,
                                                 'service.enable': mock}):
                self.assertEqual(debian_ip.build_network_settings(),
                                 [u'NETWORKING=yes\n',
                                  u'HOSTNAME=Salt\n',
                                  u'DOMAIN=saltstack.com\n'])

                mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                                 ('error'))
                with patch.object(jinja2.Environment, 'get_template', mock):
                    self.assertEqual(debian_ip.build_network_settings(), '')

        with patch.dict(debian_ip.__grains__, {'osfullname': 'Ubuntu',
                                               'osrelease': '10'}):
            mock = MagicMock(return_value=True)
            with patch.dict(debian_ip.__salt__, {'service.available': mock,
                                                 'service.disable': mock,
                                                 'service.enable': mock}):
                mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                                 ('error'))
                with patch.object(jinja2.Environment, 'get_template', mock):
                    self.assertEqual(debian_ip.build_network_settings(), '')

                with patch.object(debian_ip, '_read_temp',
                                  MagicMock(return_value=True)):
                    self.assertTrue(debian_ip.build_network_settings
                                    (test='True'))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(DebianIpTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Wrapper around uptime API
=========================
'''

# Import Python Libs
from __future__ import absolute_import
import logging

try:
    import requests
    ENABLED = True
except ImportError:
    ENABLED = False


from salt.exceptions import CommandExecutionError

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Only load this module if the requests python module is available
    '''
    if ENABLED:
        return 'uptime'
    return (False, 'uptime module needs the python requests module to work')


def create(name, **params):
    '''Create a check on a given URL.

    Additional parameters can be used and are passed to API (for
    example interval, maxTime, etc). See the documentation
    https://github.com/fzaninotto/uptime for a full list of the
    parameters.

    CLI Example:

    .. code-block:: bash

        salt '*' uptime.create http://example.org

    '''
    if check_exists(name):
        msg = 'Trying to create check that already exists : {0}'.format(name)
        log.error(msg)
        raise CommandExecutionError(msg)
    application_url = _get_application_url()
    log.debug('[uptime] trying PUT request')
    params.update(url=name)
    req = requests.put('{0}/api/checks'.format(application_url), data=params)
    if not req.ok:
        raise CommandExecutionError(
            'request to uptime failed : {0}'.format(req.reason)
        )
    log.debug('[uptime] PUT request successfull')
    return req.json()['_id']


def delete(name):
    '''
    Delete a check on a given URL

    CLI Example:

    .. code-block:: bash

        salt '*' uptime.delete http://example.org
    '''
    if not check_exists(name):
        msg = "Trying to delete check that doesn't exists : {0}".format(name)
        log.error(msg)
        raise CommandExecutionError(msg)
    application_url = _get_application_url()
    log.debug('[uptime] trying DELETE request')
    jcontent = requests.get('{0}/api/checks'.format(application_url)).json()
    url_id = [x['_id'] for x in jcontent if x['url'] == name][0]
    req = requests.delete('{0}/api/checks/{1}'.format(application_url, url_id))
    if not req.ok:
        raise CommandExecutionError(
            'request to uptime failed : {0}'.format(req.reason)
        )
    log.debug('[uptime] DELETE request successfull')
    return True


def _get_application_url():
    '''
    Helper function to get application url from pillar
    '''
    application_url = __salt__['pillar.get']('uptime:application_url')
    if application_url is None:
        log.error('Could not load {0} pillar'.format('uptime:application_url'))
        msg = '{0} pillar is required for authentication'
        raise CommandExecutionError(
            msg.format('uptime:application_url')
        )
    return application_url


def checks_list():
    '''
    List URL checked by uptime

    CLI Example:

    .. code-block:: bash

        salt '*' uptime.checks_list
    '''
    application_url = _get_application_url()
    log.debug('[uptime] get checks')
    jcontent = requests.get('{0}/api/checks'.format(application_url)).json()
    return [x['url'] for x in jcontent]


def check_exists(name):
    '''
    Check if a given URL is in being monitored by uptime

    CLI Example:

    .. code-block:: bash

        salt '*' uptime.check_exists http://example.org
    '''
    if name in checks_list():
        log.debug('[uptime] found {0} in checks'.format(name))
        return True
    return False



# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import absolute_import
import salt.client.ssh
from salt.utils import parsers
from salt.utils.verify import verify_log


class SaltSSH(parsers.SaltSSHOptionParser):
    '''
    Used to Execute the salt ssh routine
    '''

    def run(self):
        self.parse_args()
        self.setup_logfile_logger()
        verify_log(self.config)

        ssh = salt.client.ssh.SSH(self.config)
        ssh.run()



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.states import event

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

ensure_in_syspath('../../')

event.__opts__ = {}
event.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class EventTestCase(TestCase):
    '''
        Validate the event state
    '''
    def test_send(self):
        '''
            Test to send an event to the Salt Master
        '''
        with patch.dict(event.__opts__, {'test': True}):
            self.assertDictEqual(event.send("salt"),
                                 {'changes': {'data': None, 'tag': 'salt'},
                                  'comment': 'Event would have been fired',
                                  'name': 'salt',
                                  'result': None
                                  }
                                 )

        with patch.dict(event.__opts__, {'test': False}):
            mock = MagicMock(return_value=True)
            with patch.dict(event.__salt__, {'event.send': mock}):
                self.assertDictEqual(event.send("salt"),
                                     {'changes': {'data': None, 'tag': 'salt'},
                                      'comment': 'Event fired',
                                      'name': 'salt',
                                      'result': True
                                      })

    def test_wait(self):
        '''
            Test to fire an event on the Salt master
        '''
        self.assertDictEqual(event.wait("salt"),
                             {'changes': {},
                              'comment': '',
                              'name': 'salt',
                              'result': True}
                             )


if __name__ == '__main__':
    from integration import run_tests
    run_tests(EventTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
"""
Tests for the boto_sns state
"""

# Import Python libs
from __future__ import absolute_import
import re

# Import Salt Testing libs
from salttesting import skipIf
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import Salt libs
import integration

# Import 3rd-party libs
NO_BOTO_MODULE = True
BOTO_NOT_CONFIGURED = True
try:
    import boto
    NO_BOTO_MODULE = False
    try:
        boto.connect_iam()
        BOTO_NOT_CONFIGURED = False
    except boto.exception.NoAuthHandlerFound:
        pass
except ImportError:
    pass


@skipIf(
    NO_BOTO_MODULE,
    'Please install the boto library before running boto integration tests.'
)
@skipIf(
    BOTO_NOT_CONFIGURED,
    'Please setup boto AWS credentials before running boto integration tests.'
)
class BotoSNSTest(integration.ModuleCase,
                  integration.SaltReturnAssertsMixIn):

    def setUp(self):
        # The name of the topic you want to create.
        # Constraints: Topic names must be made up of only uppercase and
        # lowercase ASCII letters, numbers, underscores, and hyphens,
        # and must be between 1 and 256 characters long.
        # http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html
        self.topic_name = re.sub(r'[^a-zA-Z_-]', '_', self.id())[0:256]
        self.run_function('boto_sns.delete', name=self.topic_name)

    def tearDown(self):
        self.run_function('boto_sns.delete', name=self.topic_name)

    def test_present_new_topic_no_subscriptions(self):
        ret = self.run_state('boto_sns.present',
                             name=self.topic_name)
        self.assertSaltTrueReturn(ret)
        self.assertInSaltReturn(self.topic_name, ret, 'name')
        self.assertInSaltComment(
            'AWS SNS topic {0} created.'.format(self.topic_name),
            ret
        )
        self.assertSaltStateChangesEqual(ret,
            {'old': None, 'new': {'topic': self.topic_name, 'subscriptions': []}})

    def test_present_new_topic_with_subscriptions(self):
        ret = self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint'
                },
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint-2'
                }
            ]
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(
            ret,
            {'old': None,
             'new': {
                'topic': self.topic_name,
                'subscriptions': [
                    {'protocol': 'https',
                     'endpoint': 'https://www.example.com/sns/endpoint'
                    },
                    {'protocol': 'https',
                     'endpoint': 'https://www.example.com/sns/endpoint-2'
                    }
                ]
             }
            }
        )
        self.assertInSaltComment(
            'AWS SNS subscription https:https://www.example.com/sns/endpoint set on topic {0}.'
            .format(self.topic_name),
            ret
        )
        self.assertInSaltComment(
            'AWS SNS subscription https:https://www.example.com/sns/endpoint-2 set on topic {0}.'
            .format(self.topic_name),
            ret
        )
        self.assertSubscriptionInTopic({
            'Protocol': 'https',
            'Endpoint': 'https://www.example.com/sns/endpoint'
        }, self.topic_name)
        self.assertSubscriptionInTopic({
            'Protocol': 'https',
            'Endpoint': 'https://www.example.com/sns/endpoint-2'
        }, self.topic_name)

    def test_present_is_idempotent(self):
        self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint'
                }
            ]
        )

        ret = self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint'
                }
            ]
        )

        self.assertSaltTrueReturn(ret)
        self.assertInSaltReturn(self.topic_name, ret, 'name')
        self.assertInSaltComment(
            'AWS SNS topic {0} present.'.format(self.topic_name),
            ret
        )
        self.assertInSaltComment(
            'AWS SNS subscription https:https://www.example.com/sns/endpoint already set on topic {0}.'
            .format(self.topic_name),
            ret
        )
        self.assertSaltStateChangesEqual(ret, {})

    def test_present_add_subscription_to_existing_topic_with_no_subscription(self):
        self.run_state('boto_sns.present', name=self.topic_name)
        ret = self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint'
                }
            ]
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(
            ret,
            {'old': None,
             'new': {'subscriptions': [
                        {'protocol': 'https',
                         'endpoint': 'https://www.example.com/sns/endpoint'
                        }
                     ]
                    }
            }
        )
        self.assertInSaltComment(
            'AWS SNS subscription https:https://www.example.com/sns/endpoint set on topic {0}.'
            .format(self.topic_name),
            ret
        )

        self.assertSubscriptionInTopic({
            'Protocol': 'https',
            'Endpoint': 'https://www.example.com/sns/endpoint'
        }, self.topic_name)

    def test_present_add_new_subscription_to_existing_topic_with_subscriptions(self):
        self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint'
                }
            ]
        )
        ret = self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint-2'
                }
            ]
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(
            ret,
            {'old': None,
             'new': {
                'subscriptions': [
                    {'protocol': 'https',
                     'endpoint': 'https://www.example.com/sns/endpoint-2'
                    }
                ]
             }
            }
        )
        self.assertInSaltComment(
            'AWS SNS subscription https:https://www.example.com/sns/endpoint-2 set on topic {0}.'
            .format(self.topic_name),
            ret
        )

        self.assertSubscriptionInTopic({
            'Protocol': 'https',
            'Endpoint': 'https://www.example.com/sns/endpoint'
        }, self.topic_name)
        self.assertSubscriptionInTopic({
            'Protocol': 'https',
            'Endpoint': 'https://www.example.com/sns/endpoint-2'
        }, self.topic_name)

    def test_present_test_mode_no_subscriptions(self):
        ret = self.run_state('boto_sns.present',
                             name=self.topic_name,
                             test=True)
        self.assertSaltNoneReturn(ret)
        self.assertInSaltReturn(self.topic_name, ret, 'name')
        self.assertInSaltComment(
            'AWS SNS topic {0} is set to be created.'.format(self.topic_name),
            ret
        )
        self.assertSaltStateChangesEqual(ret, {})
        ret = self.run_function('boto_sns.exists', name=self.topic_name)
        self.assertFalse(ret)

    def test_present_test_mode_with_subscriptions(self):
        self.run_state('boto_sns.present', name=self.topic_name)
        ret = self.run_state(
            'boto_sns.present',
            name=self.topic_name,
            subscriptions=[
                {'protocol': 'https',
                 'endpoint': 'https://www.example.com/sns/endpoint'
                }
            ],
            test=True
        )
        self.assertSaltNoneReturn(ret)
        self.assertSaltStateChangesEqual(ret, {})
        self.assertInSaltComment(
            'AWS SNS subscription https:https://www.example.com/sns/endpoint to be set on topic {0}.'
            .format(self.topic_name),
            ret
        )
        ret = self.run_function(
            'boto_sns.get_all_subscriptions_by_topic',
            name=self.topic_name
        )
        self.assertEqual([], ret)

    def test_absent_not_exist(self):
        ret = self.run_state('boto_sns.absent',
                             name=self.topic_name)
        self.assertSaltTrueReturn(ret)
        self.assertInSaltReturn(self.topic_name, ret, 'name')
        self.assertInSaltComment(
            'AWS SNS topic {0} does not exist.'.format(self.topic_name),
            ret
        )
        self.assertSaltStateChangesEqual(ret, {})

    def test_absent_already_exists(self):
        self.run_state('boto_sns.present',
                       name=self.topic_name)
        ret = self.run_state('boto_sns.absent',
                             name=self.topic_name)
        self.assertSaltTrueReturn(ret)
        self.assertInSaltReturn(self.topic_name, ret, 'name')
        self.assertInSaltComment(
            'AWS SNS topic {0} does not exist.'.format(self.topic_name),
            ret
        )
        self.assertSaltStateChangesEqual(
            ret, {'new': None, 'old': {'topic': self.topic_name}})

    def test_absent_test_mode(self):
        self.run_state('boto_sns.present', name=self.topic_name)
        ret = self.run_state('boto_sns.absent',
                             name=self.topic_name,
                             test=True)
        self.assertSaltNoneReturn(ret)
        self.assertInSaltReturn(self.topic_name, ret, 'name')
        self.assertInSaltComment(
            'AWS SNS topic {0} is set to be removed.'.format(self.topic_name),
            ret
        )
        self.assertSaltStateChangesEqual(ret, {})
        ret = self.run_function('boto_sns.exists', name=self.topic_name)
        self.assertTrue(ret)

    def assertSubscriptionInTopic(self, subscription, topic_name):
        ret = self.run_function(
            'boto_sns.get_all_subscriptions_by_topic',
            name=topic_name
        )
        for _subscription in ret:
            try:
                self.assertDictContainsSubset(subscription, _subscription)
                return True
            except AssertionError:
                continue
        raise self.failureException(
            'Subscription {0} not found in topic {1} subscriptions: {2}'
            .format(subscription, topic_name, ret)
        )



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Nicole Thomas <nicole@saltstack.com>`
'''

# Import python libs
from __future__ import absolute_import
import grp
import pwd

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import mac_user
from salt.exceptions import SaltInvocationError, CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MacUserTestCase(TestCase):
    '''
    TestCase for the salt.modules.mac_user modules
    '''

    mac_user.__context__ = {}
    mac_user.__grains__ = {}
    mac_user.__salt__ = {}

    mock_pwall = [pwd.struct_passwd(('_amavisd', '*', 83, 83, 'AMaViS Daemon',
                                    '/var/virusmails', '/usr/bin/false')),
                  pwd.struct_passwd(('_appleevents', '*', 55, 55,
                                     'AppleEvents Daemon',
                                    '/var/empty', '/usr/bin/false')),
                  pwd.struct_passwd(('_appowner', '*', 87, 87,
                                     'Application Owner',
                                     '/var/empty', '/usr/bin/false'))]
    mock_pwnam = pwd.struct_passwd(('_TEST_GROUP', '*', 83, 83, 'AMaViS Daemon',
                                    '/var/virusmails', '/usr/bin/false'))
    mock_getgrgid = grp.struct_group(('_TEST_GROUP', '*', 83, []))
    mock_getgrall = [grp.struct_group(('accessibility', '*', 90, [])),
                     grp.struct_group(('admin', '*', 80, ['root', 'admin']))]
    mock_info_ret = {'shell': '/bin/bash', 'name': 'test', 'gid': 4376,
                     'groups': ['TEST_GROUP'], 'home': '/Users/foo',
                     'fullname': 'TEST USER', 'uid': 4376}

    @skipIf(True, 'Waiting on some clarifications from bug report #10594')
    def test_flush_dscl_cache(self):
        # TODO: Implement tests after clarifications come in
        pass

    def test_dscl(self):
        '''
        Tests the creation of a dscl node
        '''
        mac_mock = MagicMock(return_value={'pid': 4948,
                                           'retcode': 0,
                                           'stderr': '',
                                           'stdout': ''})
        with patch.dict(mac_user.__salt__, {'cmd.run_all': mac_mock}):
            with patch.dict(mac_user.__grains__,
                            {'kernel': 'Darwin', 'osrelease': '10.9.1',
                             'osrelease_info': (10, 9, 1)}):
                self.assertEqual(mac_user._dscl('username'), {'pid': 4948,
                                                              'retcode': 0,
                                                              'stderr': '',
                                                              'stdout': ''})

    @patch('pwd.getpwall', MagicMock(return_value=mock_pwall))
    def test_first_avail_uid(self):
        '''
        Tests the availability of the next uid
        '''
        self.assertEqual(mac_user._first_avail_uid(), 501)

    # 'add' function tests: 4
    # Only tested error handling
    # Full functionality tests covered in integration testing

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_add_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.add, 'test')

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_add_whitespace(self):
        '''
        Tests if there is whitespace in the user name
        '''
        self.assertRaises(SaltInvocationError, mac_user.add, 'foo bar')

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_add_uid_int(self):
        '''
        Tests if the uid is an int
        '''
        self.assertRaises(SaltInvocationError, mac_user.add, 'foo', 'foo')

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_add_gid_int(self):
        '''
        Tests if the gid is an int
        '''
        self.assertRaises(SaltInvocationError, mac_user.add, 'foo', 20, 'foo')

    # 'delete' function tests: 2
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    def test_delete_whitespace(self):
        '''
        Tests if there is whitespace in the user name
        '''
        self.assertRaises(SaltInvocationError, mac_user.delete, 'foo bar')

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_delete_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertTrue(mac_user.delete('foo'))

    @patch('pwd.getpwall', MagicMock(return_value=mock_pwall))
    @patch('salt.modules.mac_user.list_groups',
           MagicMock(return_value=['TEST_GROUP']))
    def test_getent(self):
        '''
        Tests the list of information for all users
        '''
        ret = [{'shell': '/usr/bin/false', 'name': '_amavisd', 'gid': 83,
                'groups': ['TEST_GROUP'], 'home': '/var/virusmails',
                'fullname': 'AMaViS Daemon', 'uid': 83},
               {'shell': '/usr/bin/false', 'name': '_appleevents', 'gid': 55,
                'groups': ['TEST_GROUP'], 'home': '/var/empty',
                'fullname': 'AppleEvents Daemon', 'uid': 55},
               {'shell': '/usr/bin/false', 'name': '_appowner', 'gid': 87,
                'groups': ['TEST_GROUP'], 'home': '/var/empty',
                'fullname': 'Application Owner', 'uid': 87}]
        self.assertEqual(mac_user.getent(), ret)

    # 'chuid' function tests: 3
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    def test_chuid_int(self):
        '''
        Tests if the uid is an int
        '''
        self.assertRaises(SaltInvocationError, mac_user.chuid, 'foo', 'foo')

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_chuid_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.chuid, 'foo', 4376)

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_chuid_same_uid(self):
        '''
        Tests if the user's uid is the same as as the argument
        '''
        self.assertTrue(mac_user.chuid('foo', 4376))

    # 'chgid' function tests: 3
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    def test_chgid_int(self):
        '''
        Tests if the gid is an int
        '''
        self.assertRaises(SaltInvocationError, mac_user.chgid, 'foo', 'foo')

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_chgid_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.chgid, 'foo', 4376)

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_chgid_same_gid(self):
        '''
        Tests if the user's gid is the same as as the argument
        '''
        self.assertTrue(mac_user.chgid('foo', 4376))

    # 'chshell' function tests: 2
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_chshell_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.chshell,
                          'foo', '/bin/bash')

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_chshell_same_shell(self):
        '''
        Tests if the user's shell is the same as the argument
        '''
        self.assertTrue(mac_user.chshell('foo', '/bin/bash'))

    # 'chhome' function tests: 2
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_chhome_user_exists(self):
        '''
        Test if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.chhome,
                          'foo', '/Users/foo')

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_chhome_same_home(self):
        '''
        Tests if the user's home is the same as the argument
        '''
        self.assertTrue(mac_user.chhome('foo', '/Users/foo'))

    # 'chfullname' function tests: 2
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_chfullname_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.chfullname,
                          'test', 'TEST USER')

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_chfullname_same_name(self):
        '''
        Tests if the user's full name is the same as the argument
        '''
        self.assertTrue(mac_user.chfullname('test', 'TEST USER'))

    # 'chgroups' function tests: 3
    # Only tested pure logic of function
    # Full functionality tests covered in integration testing

    @patch('salt.modules.mac_user.info', MagicMock(return_value={}))
    def test_chgroups_user_exists(self):
        '''
        Tests if the user exists or not
        '''
        self.assertRaises(CommandExecutionError, mac_user.chgroups,
                          'foo', 'wheel,root')

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    def test_chgroups_bad_groups(self):
        '''
        Test if there is white space in groups argument
        '''
        self.assertRaises(SaltInvocationError, mac_user.chgroups,
                          'test', 'bad group')

    @patch('salt.modules.mac_user.info', MagicMock(return_value=mock_info_ret))
    @patch('salt.modules.mac_user.list_groups',
           MagicMock(return_value=('wheel', 'root')))
    def test_chgroups_same_desired(self):
        '''
        Tests if the user's list of groups is the same as the arguments
        '''
        mock_primary = MagicMock(return_value='wheel')
        with patch.dict(mac_user.__salt__, {'file.gid_to_group': mock_primary}):
            self.assertTrue(mac_user.chgroups('test', 'wheel,root'))

    @patch('salt.modules.mac_user.list_groups',
           MagicMock(return_value=['_TEST_GROUP']))
    def test_info(self):
        '''
        Tests the return of user information
        '''
        mock_pwnam = pwd.struct_passwd(('test', '*', 0, 0, 'TEST USER',
                                        '/var/test', '/bin/bash'))
        ret = {'shell': '/bin/bash', 'name': 'test', 'gid': 0,
               'groups': ['_TEST_GROUP'], 'home': '/var/test',
               'fullname': 'TEST USER', 'uid': 0}
        with patch('pwd.getpwnam', MagicMock(return_value=mock_pwnam)):
            self.assertEqual(mac_user.info('root'), ret)

    @patch('salt.modules.mac_user.list_groups',
           MagicMock(return_value=['_TEST_GROUP']))
    def test_format_info(self):
        '''
        Tests the formatting of returned user information
        '''
        data = pwd.struct_passwd(('_TEST_GROUP', '*', 83, 83, 'AMaViS Daemon',
                                  '/var/virusmails', '/usr/bin/false'))
        ret = {'shell': '/usr/bin/false', 'name': '_TEST_GROUP', 'gid': 83,
                     'groups': ['_TEST_GROUP'], 'home': '/var/virusmails',
                     'fullname': 'AMaViS Daemon', 'uid': 83}
        self.assertEqual(mac_user._format_info(data), ret)

    @patch('pwd.getpwall', MagicMock(return_value=mock_pwall))
    def test_list_users(self):
        '''
        Tests the list of all users
        '''
        ret = ['_amavisd', '_appleevents', '_appowner']
        self.assertEqual(mac_user.list_users(), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(MacUserTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Manage users on Mac OS 10.7+

.. important::
    If you feel that Salt should be using this module to manage users on a
    minion, and it is using a different module (or gives an error similar to
    *'user.info' is not available*), see :ref:`here
    <module-provider-override>`.
'''

# Import python libs
from __future__ import absolute_import
try:
    import pwd
except ImportError:
    pass
import logging
import random
import string
import time

# Import 3rdp-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin

# Import salt libs
import salt.utils
from salt.exceptions import CommandExecutionError, SaltInvocationError
from salt.ext.six import string_types

try:
    from shlex import quote as _cmd_quote  # pylint: disable=E0611
except ImportError:
    from pipes import quote as _cmd_quote

log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = 'user'


def __virtual__():
    if (__grains__.get('kernel') != 'Darwin' or
            __grains__['osrelease_info'] < (10, 7)):
        return False
    else:
        return __virtualname__


def _flush_dscl_cache():
    '''
    Flush dscl cache
    '''
    __salt__['cmd.run']('dscacheutil -flushcache')


def _dscl(cmd, ctype='create'):
    '''
    Run a dscl -create command
    '''
    if __grains__['osrelease_info'] < (10, 8):
        source, noderoot = '.', ''
    else:
        source, noderoot = 'localhost', '/Local/Default'

    # Note, it's OK that cmd is not quoted here, we clean it up below
    return __salt__['cmd.run_all'](
        'dscl {0} -{1} {2}{3}'.format(source, ctype, noderoot, cmd),
        output_loglevel='quiet' if ctype == 'passwd' else 'warning'
    )


def _first_avail_uid():
    uids = set(x.pw_uid for x in pwd.getpwall())
    for idx in range(501, 2 ** 24):
        if idx not in uids:
            return idx


def add(name,
        uid=None,
        gid=None,
        groups=None,
        home=None,
        shell=None,
        fullname=None,
        createhome=True,
        **kwargs):
    '''
    Add a user to the minion

    CLI Example:

    .. code-block:: bash

        salt '*' user.add name <uid> <gid> <groups> <home> <shell>
    '''
    if info(name):
        raise CommandExecutionError('User {0!r} already exists'.format(name))

    if salt.utils.contains_whitespace(name):
        raise SaltInvocationError('Username cannot contain whitespace')

    if uid is None:
        uid = _first_avail_uid()
    if gid is None:
        gid = 20  # gid 20 == 'staff', the default group
    if home is None:
        home = '/Users/{0}'.format(name)
    if shell is None:
        shell = '/bin/bash'
    if fullname is None:
        fullname = ''
    # TODO: do createhome as well

    if not isinstance(uid, int):
        raise SaltInvocationError('uid must be an integer')
    if not isinstance(gid, int):
        raise SaltInvocationError('gid must be an integer')

    _dscl('/Users/{0} UniqueID {1!r}'.format(_cmd_quote(name), uid))
    _dscl('/Users/{0} PrimaryGroupID {1!r}'.format(_cmd_quote(name), gid))
    _dscl('/Users/{0} UserShell {1!r}'.format(_cmd_quote(name),
                                              _cmd_quote(shell)))
    _dscl('/Users/{0} NFSHomeDirectory {1!r}'.format(_cmd_quote(name),
                                                     _cmd_quote(home)))
    _dscl('/Users/{0} RealName {1!r}'.format(_cmd_quote(name),
                                             _cmd_quote(fullname)))

    # Set random password, since without a password the account will not be
    # available. TODO: add shadow module
    randpass = ''.join(
        random.SystemRandom().choice(string.letters + string.digits) for x in range(20)
    )
    _dscl('/Users/{0} {1!r}'.format(_cmd_quote(name),
                                    _cmd_quote(randpass)), ctype='passwd')

    # dscl buffers changes, sleep before setting group membership
    time.sleep(1)
    if groups:
        chgroups(name, groups)
    return True


def delete(name, *args):
    '''
    Remove a user from the minion

    CLI Example:

    .. code-block:: bash

        salt '*' user.delete foo
    '''
    ### NOTE: *args isn't used here but needs to be included in this function
    ### for compatibility with the user.absent state
    if salt.utils.contains_whitespace(name):
        raise SaltInvocationError('Username cannot contain whitespace')
    if not info(name):
        return True
    # Remove from any groups other than primary group. Needs to be done since
    # group membership is managed separately from users and an entry for the
    # user will persist even after the user is removed.
    chgroups(name, ())
    return _dscl('/Users/{0}'.format(_cmd_quote(name)), ctype='delete')['retcode'] == 0


def getent(refresh=False):
    '''
    Return the list of all info for all users

    CLI Example:

    .. code-block:: bash

        salt '*' user.getent
    '''
    if 'user.getent' in __context__ and not refresh:
        return __context__['user.getent']

    ret = []
    for data in pwd.getpwall():
        ret.append(_format_info(data))
    __context__['user.getent'] = ret
    return ret


def chuid(name, uid):
    '''
    Change the uid for a named user

    CLI Example:

    .. code-block:: bash

        salt '*' user.chuid foo 4376
    '''
    if not isinstance(uid, int):
        raise SaltInvocationError('uid must be an integer')
    pre_info = info(name)
    if not pre_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    if uid == pre_info['uid']:
        return True
    _dscl(
        '/Users/{0} UniqueID {1!r} {2!r}'.format(_cmd_quote(name),
                                                 pre_info['uid'],
                                                 uid),
        ctype='change'
    )
    # dscl buffers changes, sleep 1 second before checking if new value
    # matches desired value
    time.sleep(1)
    return info(name).get('uid') == uid


def chgid(name, gid):
    '''
    Change the default group of the user

    CLI Example:

    .. code-block:: bash

        salt '*' user.chgid foo 4376
    '''
    if not isinstance(gid, int):
        raise SaltInvocationError('gid must be an integer')
    pre_info = info(name)
    if not pre_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    if gid == pre_info['gid']:
        return True
    _dscl(
        '/Users/{0} PrimaryGroupID {1!r} {2!r}'.format(
            _cmd_quote(name),
            pre_info['gid'],
            gid),
        ctype='change'
    )
    # dscl buffers changes, sleep 1 second before checking if new value
    # matches desired value
    time.sleep(1)
    return info(name).get('gid') == gid


def chshell(name, shell):
    '''
    Change the default shell of the user

    CLI Example:

    .. code-block:: bash

        salt '*' user.chshell foo /bin/zsh
    '''
    pre_info = info(name)
    if not pre_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    if shell == pre_info['shell']:
        return True
    _dscl(
        '/Users/{0} UserShell {1!r} {2!r}'.format(
            _cmd_quote(name),
            _cmd_quote(pre_info['shell']),
            _cmd_quote(shell)),
        ctype='change'
    )
    # dscl buffers changes, sleep 1 second before checking if new value
    # matches desired value
    time.sleep(1)
    return info(name).get('shell') == shell


def chhome(name, home):
    '''
    Change the home directory of the user

    CLI Example:

    .. code-block:: bash

        salt '*' user.chhome foo /Users/foo
    '''
    pre_info = info(name)
    if not pre_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    if home == pre_info['home']:
        return True
    _dscl(
        '/Users/{0} NFSHomeDirectory {1!r} {2!r}'.format(
            _cmd_quote(name),
            _cmd_quote(pre_info['home']),
           _cmd_quote(home)),
        ctype='change'
    )
    # dscl buffers changes, sleep 1 second before checking if new value
    # matches desired value
    time.sleep(1)
    return info(name).get('home') == home


def chfullname(name, fullname):
    '''
    Change the user's Full Name

    CLI Example:

    .. code-block:: bash

        salt '*' user.chfullname foo 'Foo Bar'
    '''
    fullname = str(fullname)
    pre_info = info(name)
    if not pre_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    if fullname == pre_info['fullname']:
        return True
    _dscl(
        '/Users/{0} RealName {1!r}'.format(_cmd_quote(name), fullname),
        # use a "create" command, because a "change" command would fail if
        # current fullname is an empty string. The "create" will just overwrite
        # this field.
        ctype='create'
    )
    # dscl buffers changes, sleep 1 second before checking if new value
    # matches desired value
    time.sleep(1)
    return info(name).get('fullname') == fullname


def chgroups(name, groups, append=False):
    '''
    Change the groups to which the user belongs. Note that the user's primary
    group does not have to be one of the groups passed, membership in the
    user's primary group is automatically assumed.

    groups
        Groups to which the user should belong, can be passed either as a
        python list or a comma-separated string

    append
        Instead of removing user from groups not included in the ``groups``
        parameter, just add user to any groups for which they are not members

    CLI Example:

    .. code-block:: bash

        salt '*' user.chgroups foo wheel,root
    '''
    ### NOTE: **args isn't used here but needs to be included in this
    ### function for compatibility with the user.present state
    uinfo = info(name)
    if not uinfo:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    if isinstance(groups, string_types):
        groups = groups.split(',')

    bad_groups = [x for x in groups if salt.utils.contains_whitespace(x)]
    if bad_groups:
        raise SaltInvocationError(
            'Invalid group name(s): {0}'.format(', '.join(bad_groups))
        )
    ugrps = set(list_groups(name))
    desired = set(str(x) for x in groups if bool(str(x)))
    primary_group = __salt__['file.gid_to_group'](uinfo['gid'])
    if primary_group:
        desired.add(primary_group)
    if ugrps == desired:
        return True
    # Add groups from which user is missing
    for group in desired - ugrps:
        _dscl(
            '/Groups/{0} GroupMembership {1}'.format(_cmd_quote(group),
                                                     _cmd_quote(name)),
            ctype='append'
        )
    if not append:
        # Remove from extra groups
        for group in ugrps - desired:
            _dscl(
                '/Groups/{0} GroupMembership {1}'.format(_cmd_quote(group),
                                                         _cmd_quote(name)),
                ctype='delete'
            )
    time.sleep(1)
    return set(list_groups(name)) == desired


def info(name):
    '''
    Return user information

    CLI Example:

    .. code-block:: bash

        salt '*' user.info root
    '''
    try:
        data = pwd.getpwnam(name)
    except KeyError:
        return {}
    else:
        return _format_info(data)


def _format_info(data):
    '''
    Return user information in a pretty way
    '''
    return {'gid': data.pw_gid,
            'groups': list_groups(data.pw_name),
            'home': data.pw_dir,
            'name': data.pw_name,
            'shell': data.pw_shell,
            'uid': data.pw_uid,
            'fullname': data.pw_gecos}


def list_groups(name):
    '''
    Return a list of groups the named user belongs to

    CLI Example:

    .. code-block:: bash

        salt '*' user.list_groups foo
    '''
    groups = [group for group in salt.utils.get_group_list(name) if not group.startswith('_')]
    return groups


def list_users():
    '''
    Return a list of all users

    CLI Example:

    .. code-block:: bash

        salt '*' user.list_users
    '''
    return sorted([user.pw_name for user in pwd.getpwall()])


def rename(name, new_name):
    '''
    Change the username for a named user

    CLI Example:

    .. code-block:: bash

        salt '*' user.rename name new_name
    '''
    current_info = info(name)
    if not current_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    new_info = info(new_name)
    if new_info:
        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
    _dscl(
        '/Users/{0} RecordName {0!r} {2!r}'.format(name, new_name),
        ctype='change'
    )
    # dscl buffers changes, sleep 1 second before checking if new value
    # matches desired value
    time.sleep(1)
    return info(new_name).get('RecordName') == new_name



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import genesis


# Globals
genesis.__grains__ = {}
genesis.__salt__ = {}
genesis.__context__ = {}
genesis.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GenesisTestCase(TestCase):
    '''
    Test cases for salt.modules.genesis
    '''
    def test_bootstrap(self):
        '''
        Test for Create an image for a specific platform.
        '''
        mock = MagicMock(return_value=False)
        with patch.dict(genesis.__salt__, {'file.directory_exists': mock}):
            mock = MagicMock(side_effect=Exception('foo'))
            with patch.dict(genesis.__salt__, {'file.mkdir': mock}):
                self.assertEqual(genesis.bootstrap('platform', 'root'),
                                 {'Error': "Exception('foo',)"})

        with patch.object(genesis, '_bootstrap_yum', return_value='A'):
            self.assertEqual(genesis.bootstrap('rpm', 'root', 'dir1'), 'A')

        with patch.object(genesis, '_bootstrap_deb', return_value='A'):
            self.assertEqual(genesis.bootstrap('deb', 'root', 'dir1'), 'A')

        with patch.object(genesis, '_bootstrap_pacman', return_value='A'):
            self.assertEqual(genesis.bootstrap('pacman', 'root', 'dir1'), 'A')

    @patch('salt.utils.which', MagicMock(return_value=False))
    def test_avail_platforms(self):
        '''
        Test for Return which platforms are available
        '''
        self.assertFalse(genesis.avail_platforms()['deb'])

    def test_pack(self):
        '''
        Test for Pack up a directory structure, into a specific format
        '''
        with patch.object(genesis, '_tar', return_value='tar'):
            self.assertEqual(genesis.pack('name', 'root'), None)

    def test_unpack(self):
        '''
        Test for Unpack an image into a directory structure
        '''
        with patch.object(genesis, '_untar', return_value='untar'):
            self.assertEqual(genesis.unpack('name', 'root'), None)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GenesisTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Marathon
========

Proxy minion for managing a Marathon cluster.

Dependencies
------------

- :doc:`marathon execution module (salt.modules.marathon) </ref/modules/all/salt.modules.marathon>`

Pillar
------

The marathon proxy configuration requires a 'base_url' property that points to
the marathon endpoint:

.. code-block:: yaml

    proxy:
      proxytype: marathon
      base_url: http://my-marathon-master.mydomain.com:8080

.. versionadded:: 2015.8.2
'''
from __future__ import absolute_import

import logging
import salt.utils.http


__proxyenabled__ = ['marathon']
CONFIG = {}
CONFIG_BASE_URL = 'base_url'
log = logging.getLogger(__file__)


def __virtual__():
    return True


def init(opts):
    '''
    Perform any needed setup.
    '''
    if CONFIG_BASE_URL in opts['proxy']:
        CONFIG[CONFIG_BASE_URL] = opts['proxy'][CONFIG_BASE_URL]
    else:
        log.error('missing proxy property %s', CONFIG_BASE_URL)
    log.debug('CONFIG: %s', CONFIG)


def ping():
    '''
    Is the marathon api responding?
    '''
    try:
        response = salt.utils.http.query(
            "{0}/ping".format(CONFIG[CONFIG_BASE_URL]),
            decode_type='plain',
            decode=True,
        )
        log.debug(
            'marathon.info returned succesfully: %s',
            response,
        )
        if 'text' in response and response['text'].strip() == 'pong':
            return True
    except Exception as ex:
        log.error(
            'error calling marathon.info with base_url %s: %s',
            CONFIG[CONFIG_BASE_URL],
            ex,
        )
    return False


def shutdown(opts):
    '''
    For this proxy shutdown is a no-op
    '''
    log.debug('marathon proxy shutdown() called...')



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Bo Maryniuk <bo@suse.de>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    Mock,
    MagicMock,
    call,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
from salt.exceptions import CommandExecutionError

import os
from salt.ext.six.moves import configparser
import StringIO

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')


class ZyppCallMock(object):
    def __init__(self, return_value=None):
        self.__return_value = return_value

    def __getattr__(self, item):
        return self

    def __call__(self, *args, **kwargs):
        return MagicMock(return_value=self.__return_value)()


def get_test_data(filename):
    '''
    Return static test data
    '''
    return open(os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'zypp'), filename)).read()


# Import Salt Libs
from salt.modules import zypper

# Globals
zypper.__salt__ = dict()
zypper.__context__ = dict()
zypper.rpm = None


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ZypperTestCase(TestCase):

    '''
    Test cases for salt.modules.zypper
    '''

    def setUp(self):
        self.new_repo_config = dict(
            name='mock-repo-name',
            url='http://repo.url/some/path'
        )
        side_effect = [
            Mock(**{'sections.return_value': []}),
            Mock(**{'sections.return_value': [self.new_repo_config['name']]})
        ]
        self.zypper_patcher_config = {
            '_get_configured_repos': Mock(side_effect=side_effect),
            '__zypper__': Mock(),
            'get_repo': Mock(return_value={})
        }

    def test_list_upgrades(self):
        '''
        List package upgrades
        :return:
        '''
        ref_out = {
            'stdout': get_test_data('zypper-updates.xml'),
            'stderr': None,
            'retcode': 0
        }
        with patch.dict(zypper.__salt__, {'cmd.run_all': MagicMock(return_value=ref_out)}):
            upgrades = zypper.list_upgrades(refresh=False)
            self.assertEqual(len(upgrades), 3)
            for pkg, version in {'SUSEConnect': '0.2.33-7.1',
                                 'bind-utils': '9.9.6P1-35.1',
                                 'bind-libs': '9.9.6P1-35.1'}.items():
                self.assertIn(pkg, upgrades)
                self.assertEqual(upgrades[pkg], version)

    def test_zypper_caller(self):
        '''
        Test Zypper caller.
        :return:
        '''
        class RunSniffer(object):
            def __init__(self, stdout=None, stderr=None, retcode=None):
                self.calls = list()
                self._stdout = stdout or ''
                self._stderr = stderr or ''
                self._retcode = retcode or 0

            def __call__(self, *args, **kwargs):
                self.calls.append({'args': args, 'kwargs': kwargs})
                return {'stdout': self._stdout,
                        'stderr': self._stderr,
                        'retcode': self._retcode}

        stdout_xml_snippet = '<?xml version="1.0"?><test foo="bar"/>'
        sniffer = RunSniffer(stdout=stdout_xml_snippet)
        with patch.dict('salt.modules.zypper.__salt__', {'cmd.run_all': sniffer}):
            self.assertEqual(zypper.__zypper__.call('foo'), stdout_xml_snippet)
            self.assertEqual(len(sniffer.calls), 1)

            zypper.__zypper__.call('bar')
            self.assertEqual(len(sniffer.calls), 2)
            self.assertEqual(sniffer.calls[0]['args'][0], ['zypper', '--non-interactive', '--no-refresh', 'foo'])
            self.assertEqual(sniffer.calls[1]['args'][0], ['zypper', '--non-interactive', '--no-refresh', 'bar'])

            dom = zypper.__zypper__.xml.call('xml-test')
            self.assertEqual(sniffer.calls[2]['args'][0], ['zypper', '--non-interactive', '--xmlout',
                                                           '--no-refresh', 'xml-test'])
            self.assertEqual(dom.getElementsByTagName('test')[0].getAttribute('foo'), 'bar')

            zypper.__zypper__.refreshable.call('refresh-test')
            self.assertEqual(sniffer.calls[3]['args'][0], ['zypper', '--non-interactive', 'refresh-test'])

            zypper.__zypper__.nolock.call('no-locking-test')
            self.assertEqual(sniffer.calls[4].get('kwargs', {}).get('env', {}).get('ZYPP_READONLY_HACK'), "1")
            self.assertEqual(sniffer.calls[4].get('kwargs', {}).get('env', {}).get('SALT_RUNNING'), "1")

            zypper.__zypper__.call('locking-test')
            self.assertEqual(sniffer.calls[5].get('kwargs', {}).get('env', {}).get('ZYPP_READONLY_HACK'), None)
            self.assertEqual(sniffer.calls[5].get('kwargs', {}).get('env', {}).get('SALT_RUNNING'), "1")

        # Test exceptions
        stdout_xml_snippet = '<?xml version="1.0"?><stream><message type="error">Booya!</message></stream>'
        sniffer = RunSniffer(stdout=stdout_xml_snippet, retcode=1)
        with patch.dict('salt.modules.zypper.__salt__', {'cmd.run_all': sniffer}):
            with self.assertRaisesRegexp(CommandExecutionError, '^Zypper command failure: Booya!$'):
                zypper.__zypper__.xml.call('crashme')

            with self.assertRaisesRegexp(CommandExecutionError, "^Zypper command failure: Check Zypper's logs.$"):
                zypper.__zypper__.call('crashme again')

            zypper.__zypper__.noraise.call('stay quiet')
            self.assertEqual(zypper.__zypper__.error_msg, "Check Zypper's logs.")

    def test_list_upgrades_error_handling(self):
        '''
        Test error handling in the list package upgrades.
        :return:
        '''
        # Test handled errors
        ref_out = {
            'stdout': '''<?xml version='1.0'?>
<stream>
 <message type="info">Refreshing service &apos;container-suseconnect&apos;.</message>
 <message type="error">Some handled zypper internal error</message>
 <message type="error">Another zypper internal error</message>
</stream>
            ''',
            'stderr': '',
            'retcode': 1,
        }
        with patch.dict('salt.modules.zypper.__salt__', {'cmd.run_all': MagicMock(return_value=ref_out)}):
            with self.assertRaisesRegexp(CommandExecutionError,
                    "^Zypper command failure: Some handled zypper internal error\nAnother zypper internal error$"):
                zypper.list_upgrades(refresh=False)

        # Test unhandled error
        ref_out = {
            'retcode': 1,
            'stdout': '',
            'stderr': ''
        }
        with patch.dict('salt.modules.zypper.__salt__', {'cmd.run_all': MagicMock(return_value=ref_out)}):
            with self.assertRaisesRegexp(CommandExecutionError, "^Zypper command failure: Check Zypper's logs.$"):
                zypper.list_upgrades(refresh=False)

    def test_list_products(self):
        '''
        List products test.
        '''
        for filename, test_data in {
            'zypper-products-sle12sp1.xml': {
                'name': ['SLES', 'SLES', 'SUSE-Manager-Proxy',
                         'SUSE-Manager-Server', 'sle-manager-tools-beta',
                         'sle-manager-tools-beta-broken-eol', 'sle-manager-tools-beta-no-eol'],
                'vendor': 'SUSE LLC <https://www.suse.com/>',
                'release': ['0', '0', '0', '0', '0', '0', '0'],
                'productline': [None, None, None, None, None, None, 'sles'],
                'eol_t': [None, 0, 1509408000, 1522454400, 1522454400, 1730332800, 1730332800],
                'isbase': [False, False, False, False, False, False, True],
                'installed': [False, False, False, False, False, False, True],
                'registerrelease': [None, None, None, None, None, None, '123'],
            },
            'zypper-products-sle11sp3.xml': {
                'name': ['SUSE-Manager-Server', 'SUSE-Manager-Server', 'SUSE-Manager-Server-Broken-EOL',
                         'SUSE_SLES', 'SUSE_SLES', 'SUSE_SLES', 'SUSE_SLES-SP4-migration'],
                'vendor': 'SUSE LINUX Products GmbH, Nuernberg, Germany',
                'release': ['1.138', '1.2', '1.2', '1.2', '1.201', '1.201', '1.4'],
                'productline': [None, None, None, None, None, 'manager', 'manager'],
                'eol_t': [None, 0, 0, 0, 0, 0, 0],
                'isbase': [False, False, False, False, False, True, True],
                'installed': [False, False, False, False, False, True, True],
                'registerrelease': [None, None, None, None, None, None, "42"],
            }}.items():

            ref_out = {
                    'retcode': 0,
                    'stdout': get_test_data(filename)
            }

            with patch.dict(zypper.__salt__, {'cmd.run_all': MagicMock(return_value=ref_out)}):
                products = zypper.list_products()
                self.assertEqual(len(products), 7)
                self.assertIn(test_data['vendor'], [product['vendor'] for product in products])
                for kwd in ['name', 'isbase', 'installed', 'release', 'productline', 'eol_t', 'registerrelease']:
                    self.assertEqual(test_data[kwd], sorted([prod.get(kwd) for prod in products]))

    def test_refresh_db(self):
        '''
        Test if refresh DB handled correctly
        '''
        ref_out = [
            "Repository 'openSUSE-Leap-42.1-LATEST' is up to date.",
            "Repository 'openSUSE-Leap-42.1-Update' is up to date.",
            "Retrieving repository 'openSUSE-Leap-42.1-Update-Non-Oss' metadata",
            "Forcing building of repository cache",
            "Building repository 'openSUSE-Leap-42.1-Update-Non-Oss' cache ..........[done]",
            "Building repository 'salt-dev' cache",
            "All repositories have been refreshed."
        ]

        run_out = {
            'stderr': '', 'stdout': '\n'.join(ref_out), 'retcode': 0
        }

        with patch.dict(zypper.__salt__, {'cmd.run_all': MagicMock(return_value=run_out)}):
            result = zypper.refresh_db()
            self.assertEqual(result.get("openSUSE-Leap-42.1-LATEST"), False)
            self.assertEqual(result.get("openSUSE-Leap-42.1-Update"), False)
            self.assertEqual(result.get("openSUSE-Leap-42.1-Update-Non-Oss"), True)

    def test_info_installed(self):
        '''
        Test the return information of the named package(s), installed on the system.

        :return:
        '''
        run_out = {
            'virgo-dummy':
                {'build_date': '2015-07-09T10:55:19Z',
                 'vendor': 'openSUSE Build Service',
                 'description': 'This is the Virgo dummy package used for testing SUSE Manager',
                 'license': 'GPL-2.0', 'build_host': 'sheep05', 'url': 'http://www.suse.com',
                 'build_date_time_t': 1436432119, 'relocations': '(not relocatable)',
                 'source_rpm': 'virgo-dummy-1.0-1.1.src.rpm', 'install_date': '2016-02-23T16:31:57Z',
                 'install_date_time_t': 1456241517, 'summary': 'Virgo dummy package', 'version': '1.0',
                 'signature': 'DSA/SHA1, Thu Jul  9 08:55:33 2015, Key ID 27fa41bd8a7c64f9',
                 'release': '1.1', 'group': 'Applications/System', 'arch': 'noarch', 'size': '17992'},

            'libopenssl1_0_0':
                {'build_date': '2015-11-04T23:20:34Z', 'vendor': 'SUSE LLC <https://www.suse.com/>',
                 'description': 'The OpenSSL Project is a collaborative effort.',
                 'license': 'OpenSSL', 'build_host': 'sheep11', 'url': 'https://www.openssl.org/',
                 'build_date_time_t': 1446675634, 'relocations': '(not relocatable)',
                 'source_rpm': 'openssl-1.0.1i-34.1.src.rpm', 'install_date': '2016-02-23T16:31:35Z',
                 'install_date_time_t': 1456241495, 'summary': 'Secure Sockets and Transport Layer Security',
                 'version': '1.0.1i', 'signature': 'RSA/SHA256, Wed Nov  4 22:21:34 2015, Key ID 70af9e8139db7c82',
                 'release': '34.1', 'group': 'Productivity/Networking/Security', 'packager': 'https://www.suse.com/',
                 'arch': 'x86_64', 'size': '2576912'},
        }
        with patch.dict(zypper.__salt__, {'lowpkg.info': MagicMock(return_value=run_out)}):
            installed = zypper.info_installed()
            # Test overall products length
            self.assertEqual(len(installed), 2)

            # Test translated fields
            for pkg_name, pkg_info in installed.items():
                self.assertEqual(installed[pkg_name].get('source'), run_out[pkg_name]['source_rpm'])

            # Test keys transition from the lowpkg.info
            for pn_key, pn_val in run_out['virgo-dummy'].items():
                if pn_key == 'source_rpm':
                    continue
                self.assertEqual(installed['virgo-dummy'][pn_key], pn_val)

    def test_info_available(self):
        '''
        Test return the information of the named package available for the system.

        :return:
        '''
        test_pkgs = ['vim', 'emacs', 'python']
        with patch('salt.modules.zypper.__zypper__', ZyppCallMock(return_value=get_test_data('zypper-available.txt'))):
            available = zypper.info_available(*test_pkgs, refresh=False)
            self.assertEqual(len(available), 3)
            for pkg_name, pkg_info in available.items():
                self.assertIn(pkg_name, test_pkgs)

            self.assertEqual(available['emacs']['status'], 'up-to-date')
            self.assertTrue(available['emacs']['installed'])
            self.assertEqual(available['emacs']['support level'], 'Level 3')
            self.assertEqual(available['emacs']['vendor'], 'SUSE LLC <https://www.suse.com/>')
            self.assertEqual(available['emacs']['summary'], 'GNU Emacs Base Package')

            self.assertEqual(available['vim']['status'], 'not installed')
            self.assertFalse(available['vim']['installed'])
            self.assertEqual(available['vim']['support level'], 'Level 3')
            self.assertEqual(available['vim']['vendor'], 'SUSE LLC <https://www.suse.com/>')
            self.assertEqual(available['vim']['summary'], 'Vi IMproved')

    @patch('salt.modules.zypper.refresh_db', MagicMock(return_value=True))
    def test_latest_version(self):
        '''
        Test the latest version of the named package available for upgrade or installation.

        :return:
        '''
        with patch('salt.modules.zypper.__zypper__', ZyppCallMock(return_value=get_test_data('zypper-available.txt'))):
            self.assertEqual(zypper.latest_version('vim'), '7.4.326-2.62')

    @patch('salt.modules.zypper.refresh_db', MagicMock(return_value=True))
    def test_upgrade_available(self):
        '''
        Test whether or not an upgrade is available for a given package.

        :return:
        '''
        ref_out = get_test_data('zypper-available.txt')
        with patch('salt.modules.zypper.__zypper__', ZyppCallMock(return_value=get_test_data('zypper-available.txt'))):
            for pkg_name in ['emacs', 'python']:
                self.assertFalse(zypper.upgrade_available(pkg_name))
            self.assertTrue(zypper.upgrade_available('vim'))

    def test_list_pkgs(self):
        '''
        Test packages listing.

        :return:
        '''
        def _add_data(data, key, value):
            data[key] = value

        rpm_out = [
            'protobuf-java_|-2.6.1_|-3.1.develHead_|-',
            'yast2-ftp-server_|-3.1.8_|-8.1_|-',
            'jose4j_|-0.4.4_|-2.1.develHead_|-',
            'apache-commons-cli_|-1.2_|-1.233_|-',
            'jakarta-commons-discovery_|-0.4_|-129.686_|-',
            'susemanager-build-keys-web_|-12.0_|-5.1.develHead_|-',
        ]
        with patch.dict(zypper.__salt__, {'cmd.run': MagicMock(return_value=os.linesep.join(rpm_out))}):
            with patch.dict(zypper.__salt__, {'pkg_resource.add_pkg': _add_data}):
                with patch.dict(zypper.__salt__, {'pkg_resource.sort_pkglist': MagicMock()}):
                    with patch.dict(zypper.__salt__, {'pkg_resource.stringify': MagicMock()}):
                        pkgs = zypper.list_pkgs()
                        for pkg_name, pkg_version in {
                            'jakarta-commons-discovery': '0.4-129.686',
                            'yast2-ftp-server': '3.1.8-8.1',
                            'protobuf-java': '2.6.1-3.1.develHead',
                            'susemanager-build-keys-web': '12.0-5.1.develHead',
                            'apache-commons-cli': '1.2-1.233',
                            'jose4j': '0.4.4-2.1.develHead'}.items():
                            self.assertTrue(pkgs.get(pkg_name))
                            self.assertEqual(pkgs[pkg_name], pkg_version)

    def test_download(self):
        '''
        Test package download
        :return:
        '''
        download_out = {
            'stdout': get_test_data('zypper-download.xml'),
            'stderr': None,
            'retcode': 0
        }

        test_out = {
            'nmap': {
                'path': u'/var/cache/zypp/packages/SLE-12-x86_64-Pool/x86_64/nmap-6.46-1.72.x86_64.rpm',
                'repository-alias': u'SLE-12-x86_64-Pool',
                'repository-name': u'SLE-12-x86_64-Pool'
            }
        }

        with patch.dict(zypper.__salt__, {'cmd.run_all': MagicMock(return_value=download_out)}):
            with patch.dict(zypper.__salt__, {'lowpkg.checksum': MagicMock(return_value=True)}):
                self.assertEqual(zypper.download("nmap"), test_out)
                test_out['_error'] = "The following package(s) failed to download: foo"
                self.assertEqual(zypper.download("nmap", "foo"), test_out)

    def test_remove_purge(self):
        '''
        Test package removal
        :return:
        '''
        class ListPackages(object):
            def __init__(self):
                self._packages = ['vim', 'pico']
                self._pkgs = {
                    'vim': '0.18.0',
                    'emacs': '24.0.1',
                    'pico': '0.1.1',
                }

            def __call__(self):
                pkgs = self._pkgs.copy()
                for target in self._packages:
                    if self._pkgs.get(target):
                        del self._pkgs[target]

                return pkgs

        parsed_targets = [{'vim': None, 'pico': None}, None]
        cmd_out = {
                'retcode': 0,
                'stdout': '',
                'stderr': ''
        }

        # If config.get starts being used elsewhere, we'll need to write a
        # side_effect function.
        patches = {
            'cmd.run_all': MagicMock(return_value=cmd_out),
            'pkg_resource.parse_targets': MagicMock(return_value=parsed_targets),
            'pkg_resource.stringify': MagicMock(),
            'config.get': MagicMock(return_value=True)
        }

        with patch.dict(zypper.__salt__, patches):
            with patch('salt.modules.zypper.list_pkgs', ListPackages()):
                diff = zypper.remove(name='vim,pico')
                for pkg_name in ['vim', 'pico']:
                    self.assertTrue(diff.get(pkg_name))
                    self.assertTrue(diff[pkg_name]['old'])
                    self.assertFalse(diff[pkg_name]['new'])

    def test_repo_value_info(self):
        '''
        Tests if repo info is properly parsed.

        :return:
        '''
        repos_cfg = configparser.ConfigParser()
        for cfg in ['zypper-repo-1.cfg', 'zypper-repo-2.cfg']:
            repos_cfg.readfp(StringIO.StringIO(get_test_data(cfg)))

        for alias in repos_cfg.sections():
            r_info = zypper._get_repo_info(alias, repos_cfg=repos_cfg)
            self.assertEqual(type(r_info['type']), type(None))
            self.assertEqual(type(r_info['enabled']), bool)
            self.assertEqual(type(r_info['autorefresh']), bool)
            self.assertEqual(type(r_info['baseurl']), str)
            self.assertEqual(r_info['type'], None)
            self.assertEqual(r_info['enabled'], alias == 'SLE12-SP1-x86_64-Update')
            self.assertEqual(r_info['autorefresh'], alias == 'SLE12-SP1-x86_64-Update')

    def test_repo_add_nomod_noref(self):
        '''
        Test mod_repo adds the new repo and nothing else

        :return:
        '''
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        with zypper_patcher:
            zypper.mod_repo(name, **{'url': url})
            self.assertEqual(
                zypper.__zypper__.xml.call.call_args_list,
                [call('ar', url, name)]
            )
            zypper.__zypper__.refreshable.xml.call.assert_not_called()

    def test_repo_noadd_nomod_noref(self):
        '''
        Test mod_repo detects the repo already exists,
        no modification was requested and no refresh requested either

        :return:
        '''
        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        self.zypper_patcher_config['_get_configured_repos'] = Mock(
            **{'return_value.sections.return_value': [name]}
        )
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        with zypper_patcher:
            self.assertEqual(zypper.mod_repo(name, **{'url': url}),
                             {'comment': 'Specified arguments did not result in modification of repo'})
            zypper.__zypper__.xml.call.assert_not_called()
            zypper.__zypper__.refreshable.xml.call.assert_not_called()

    def test_repo_add_mod_noref(self):
        '''
        Test mod_repo adds the new repo and call modify to update autorefresh

        :return:
        '''
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        with zypper_patcher:
            zypper.mod_repo(name, **{'url': url, 'refresh': True})
            self.assertEqual(
                zypper.__zypper__.xml.call.call_args_list,
                [call('ar', url, name)]
            )
            zypper.__zypper__.refreshable.xml.call.assert_called_once_with(
                'mr', '--refresh', name
            )

    def test_repo_noadd_mod_noref(self):
        '''
        Test mod_repo detects the repository exists,
        calls modify to update 'autorefresh' but does not call refresh

        :return:
        '''
        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        self.zypper_patcher_config['_get_configured_repos'] = Mock(
            **{'return_value.sections.return_value': [name]})
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)
        with zypper_patcher:
            zypper.mod_repo(name, **{'url': url, 'refresh': True})
            zypper.__zypper__.xml.call.assert_not_called()
            zypper.__zypper__.refreshable.xml.call.assert_called_once_with(
                'mr', '--refresh', name
            )

    def test_repo_add_nomod_ref(self):
        '''
        Test mod_repo adds the new repo and refreshes the repo with
            `zypper --gpg-auto-import-keys refresh <repo-name>`

        :return:
        '''
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        with zypper_patcher:
            zypper.mod_repo(name, **{'url': url, 'gpgautoimport': True})
            self.assertEqual(
                zypper.__zypper__.xml.call.call_args_list,
                [
                    call('ar', url, name),
                    call('--gpg-auto-import-keys', 'refresh', name)
                ]
            )
            zypper.__zypper__.refreshable.xml.call.assert_not_called()

    def test_repo_noadd_nomod_ref(self):
        '''
        Test mod_repo detects the repo already exists,
        has nothing to modify and refreshes the repo with
            `zypper --gpg-auto-import-keys refresh <repo-name>`

        :return:
        '''
        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        self.zypper_patcher_config['_get_configured_repos'] = Mock(
            **{'return_value.sections.return_value': [name]}
        )
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        with zypper_patcher:
            zypper.mod_repo(name, **{'url': url, 'gpgautoimport': True})
            self.assertEqual(
                zypper.__zypper__.xml.call.call_args_list,
                [call('--gpg-auto-import-keys', 'refresh', name)]
            )
            zypper.__zypper__.refreshable.xml.call.assert_not_called()

    def test_repo_add_mod_ref(self):
        '''
        Test mod_repo adds the new repo,
        calls modify to update 'autorefresh' and refreshes the repo with
            `zypper --gpg-auto-import-keys refresh <repo-name>`

        :return:
        '''
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        with zypper_patcher:
            zypper.mod_repo(
                name,
                **{'url': url, 'refresh': True, 'gpgautoimport': True}
            )
            self.assertEqual(
                zypper.__zypper__.xml.call.call_args_list,
                [
                    call('ar', url, name),
                    call('--gpg-auto-import-keys', 'refresh', name)
                ]
            )
            zypper.__zypper__.refreshable.xml.call.assert_called_once_with(
                '--gpg-auto-import-keys', 'mr', '--refresh', name
            )

    def test_repo_noadd_mod_ref(self):
        '''
        Test mod_repo detects the repo already exists,
        calls modify to update 'autorefresh' and refreshes the repo with
            `zypper --gpg-auto-import-keys refresh <repo-name>`

        :return:
        '''
        url = self.new_repo_config['url']
        name = self.new_repo_config['name']
        self.zypper_patcher_config['_get_configured_repos'] = Mock(
            **{'return_value.sections.return_value': [name]}
        )
        zypper_patcher = patch.multiple(
            'salt.modules.zypper', **self.zypper_patcher_config)

        with zypper_patcher:
            zypper.mod_repo(
                name,
                **{'url': url, 'refresh': True, 'gpgautoimport': True}
            )
            self.assertEqual(
                zypper.__zypper__.xml.call.call_args_list,
                [call('--gpg-auto-import-keys', 'refresh', name)]
            )
            zypper.__zypper__.refreshable.xml.call.assert_called_once_with(
                '--gpg-auto-import-keys', 'mr', '--refresh', name
            )

if __name__ == '__main__':
    from integration import run_tests
    run_tests(ZypperTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Nicole Thomas <nicole@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.modules import darwin_sysctl
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    mock_open,
    patch,
    call,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Globals
darwin_sysctl.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DarwinSysctlTestCase(TestCase):
    '''
    TestCase for salt.modules.darwin_sysctl module
    '''

    def test_get(self):
        '''
        Tests the return of get function
        '''
        mock_cmd = MagicMock(return_value='foo')
        with patch.dict(darwin_sysctl.__salt__, {'cmd.run': mock_cmd}):
            self.assertEqual(darwin_sysctl.get('kern.ostype'), 'foo')

    def test_assign_cmd_failed(self):
        '''
        Tests if the assignment was successful or not
        '''
        cmd = {'pid': 3548, 'retcode': 1, 'stderr': '',
               'stdout': 'net.inet.icmp.icmplim: 250 -> 50'}
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(darwin_sysctl.__salt__, {'cmd.run_all': mock_cmd}):
            self.assertRaises(CommandExecutionError,
                              darwin_sysctl.assign,
                              'net.inet.icmp.icmplim', 50)

    def test_assign(self):
        '''
        Tests the return of successful assign function
        '''
        cmd = {'pid': 3548, 'retcode': 0, 'stderr': '',
               'stdout': 'net.inet.icmp.icmplim: 250 -> 50'}
        ret = {'net.inet.icmp.icmplim': '50'}
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(darwin_sysctl.__salt__, {'cmd.run_all': mock_cmd}):
            self.assertEqual(darwin_sysctl.assign(
                'net.inet.icmp.icmplim', 50), ret)

    @patch('os.path.isfile', MagicMock(return_value=False))
    def test_persist_no_conf_failure(self):
        '''
        Tests adding of config file failure
        '''
        with patch('salt.utils.fopen', mock_open()) as m_open:
            helper_open = m_open()
            helper_open.write.assertRaises(CommandExecutionError,
                                           darwin_sysctl.persist,
                                           'net.inet.icmp.icmplim',
                                           50, config=None)

    @patch('os.path.isfile', MagicMock(return_value=False))
    def test_persist_no_conf_success(self):
        '''
        Tests successful add of config file when previously not one
        '''
        with patch('salt.utils.fopen', mock_open()) as m_open:
            darwin_sysctl.persist('net.inet.icmp.icmplim', 50)
            helper_open = m_open()
            helper_open.write.assert_called_once_with(
                '#\n# Kernel sysctl configuration\n#\n')

    @patch('os.path.isfile', MagicMock(return_value=True))
    def test_persist_success(self):
        '''
        Tests successful write to existing sysctl file
        '''
        to_write = '#\n# Kernel sysctl configuration\n#\n'
        m_calls_list = [call.writelines(['net.inet.icmp.icmplim=50', '\n'])]
        with patch('salt.utils.fopen', mock_open(read_data=to_write)) as m_open:
            darwin_sysctl.persist('net.inet.icmp.icmplim', 50, config=to_write)
            helper_open = m_open()
            calls_list = helper_open.method_calls
            self.assertEqual(calls_list, m_calls_list)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(DarwinSysctlTestCase, needs_daemon=False)
